@@ -1,18 +1,13 @@
-"""
-Tests the functionality of the Blend CloudMan API. These tests require working
-credentials to supported cloud infrastructure.
-"""
 import uuid
 
 from . import GalaxyTestBase
 
 
 class TestGalaxyRoles(GalaxyTestBase.GalaxyTestBase):
-
     def setUp(self):
         super().setUp()
         self.name = f"test_{uuid.uuid4().hex}"
-        self.description = 'automated test role'
+        self.description = "automated test role"
         self.role = self.gi.roles.create_role(self.name, self.description)
 
     def tearDown(self):
@@ -22,10 +17,10 @@ class TestGalaxyRoles(GalaxyTestBase.GalaxyTestBase):
     def test_get_roles(self):
         roles = self.gi.roles.get_roles()
         for role in roles:
-            self.assertIsNotNone(role['id'])
-            self.assertIsNotNone(role['name'])
+            assert role["id"] is not None
+            assert role["name"] is not None
 
     def test_create_role(self):
-        self.assertEqual(self.role['name'], self.name)
-        self.assertEqual(self.role['description'], self.description)
-        self.assertIsNotNone(self.role['id'])
+        assert self.role["name"] == self.name
+        assert self.role["description"] == self.description
+        assert self.role["id"] is not None
