@@ -1,6 +1,8 @@
 """
 A base representation of an instance of Galaxy
 """
+from typing import Optional
+
 from bioblend.galaxy import (
     config,
     dataset_collections,
@@ -25,12 +27,18 @@ from bioblend.galaxy import (
     visual,
     workflows,
 )
-from bioblend.galaxy.client import Client
 from bioblend.galaxyclient import GalaxyClient
 
 
 class GalaxyInstance(GalaxyClient):
-    def __init__(self, url, key=None, email=None, password=None, verify=True):
+    def __init__(
+        self,
+        url: str,
+        key: Optional[str] = None,
+        email: Optional[str] = None,
+        password: Optional[str] = None,
+        verify: bool = True,
+    ) -> None:
         """
         A base representation of a connection to a Galaxy instance, identified
         by the server URL and user credentials.
@@ -97,23 +105,7 @@ class GalaxyInstance(GalaxyClient):
         self.folders = folders.FoldersClient(self)
         self.tool_dependencies = tool_dependencies.ToolDependenciesClient(self)
 
-    @property
-    def max_get_attempts(self):
-        return Client.max_get_retries()
-
-    @max_get_attempts.setter
-    def max_get_attempts(self, v):
-        Client.set_max_get_retries(v)
-
-    @property
-    def get_retry_delay(self):
-        return Client.get_retry_delay()
-
-    @get_retry_delay.setter
-    def get_retry_delay(self, v):
-        Client.set_get_retry_delay(v)
-
-    def __repr__(self):
+    def __repr__(self) -> str:
         """
         A nicer representation of this GalaxyInstance object
         """
