@@ -1,5 +1,13 @@
 import logging
 import time
+from typing import (
+    Any,
+    Dict,
+    List,
+    Optional,
+    TYPE_CHECKING,
+    Union,
+)
 
 from bioblend import (
     CHUNK_SIZE,
@@ -8,92 +16,97 @@ from bioblend import (
 from bioblend.galaxy.client import Client
 from bioblend.galaxy.datasets import TERMINAL_STATES
 
+if TYPE_CHECKING:
+    from bioblend.galaxy import GalaxyInstance
+
 log = logging.getLogger(__name__)
 
 
 class HasElements:
-
-    def __init__(self, name, type="list", elements=None):
+    def __init__(
+        self,
+        name: str,
+        type: str = "list",
+        elements: Optional[Union[List[Union["CollectionElement", "SimpleElement"]], Dict[str, Any]]] = None,
+    ) -> None:
         self.name = name
         self.type = type
         if isinstance(elements, dict):
-            self.elements = [dict(name=key, id=value, src="hda") for key, value in elements.values()]
+            self.elements: List[Union["CollectionElement", "SimpleElement"]] = [
+                HistoryDatasetElement(name=key, id=value) for key, value in elements.values()
+            ]
         elif elements:
             self.elements = elements
 
-    def add(self, element):
+    def add(self, element: Union["CollectionElement", "SimpleElement"]) -> "HasElements":
         self.elements.append(element)
         return self
 
 
 class CollectionDescription(HasElements):
-
-    def to_dict(self):
-        return dict(
-            name=self.name,
-            collection_type=self.type,
-            element_identifiers=[e.to_dict() for e in self.elements]
-        )
+    def to_dict(self) -> Dict[str, Union[str, List]]:
+        return dict(name=self.name, collection_type=self.type, element_identifiers=[e.to_dict() for e in self.elements])
 
 
 class CollectionElement(HasElements):
-
-    def to_dict(self):
+    def to_dict(self) -> Dict[str, Union[str, List]]:
         return dict(
             src="new_collection",
             name=self.name,
             collection_type=self.type,
-            element_identifiers=[e.to_dict() for e in self.elements]
+            element_identifiers=[e.to_dict() for e in self.elements],
         )
 
 
 class SimpleElement:
-
-    def __init__(self, value):
+    def __init__(self, value: Dict[str, str]) -> None:
         self.value = value
 
-    def to_dict(self):
+    def to_dict(self) -> Dict[str, str]:
         return self.value
 
 
 class HistoryDatasetElement(SimpleElement):
-
-    def __init__(self, name, id):
-        super().__init__(dict(
-            name=name,
-            src="hda",
-            id=id,
-        ))
+    def __init__(self, name: str, id: str) -> None:
+        super().__init__(
+            dict(
+                name=name,
+                src="hda",
+                id=id,
+            )
+        )
 
 
 class HistoryDatasetCollectionElement(SimpleElement):
-
-    def __init__(self, name, id):
-        super().__init__(dict(
-            name=name,
-            src="hdca",
-            id=id,
-        ))
+    def __init__(self, name: str, id: str) -> None:
+        super().__init__(
+            dict(
+                name=name,
+                src="hdca",
+                id=id,
+            )
+        )
 
 
 class LibraryDatasetElement(SimpleElement):
-
-    def __init__(self, name, id):
-        super().__init__(dict(
-            name=name,
-            src="ldda",
-            id=id,
-        ))
+    def __init__(self, name: str, id: str) -> None:
+        super().__init__(
+            dict(
+                name=name,
+                src="ldda",
+                id=id,
+            )
+        )
 
 
 class DatasetCollectionClient(Client):
-    module = 'dataset_collections'
+    gi: "GalaxyInstance"
+    module = "dataset_collections"
 
-    def __init__(self, galaxy_instance):
+    def __init__(self, galaxy_instance: "GalaxyInstance") -> None:
         super().__init__(galaxy_instance)
 
-    def show_dataset_collection(self, dataset_collection_id: str,
-                                instance_type: str = 'history') -> dict:
+    def show_dataset_collection(self, dataset_collection_id: str, instance_type: str = "history") -> Dict[str, Any]:
         """
         Get details of a given dataset collection of the current user
 
@@ -107,13 +120,12 @@ class DatasetCollectionClient(Client):
         :return: element view of the dataset collection
         """
         params = {
-            'id': dataset_collection_id,
-            'instance_type': instance_type,
+            "instance_type": instance_type,
         }
         url = self._make_url(module_id=dataset_collection_id)
-        return self._get(url=url, params=params)
+        return self._get(id=dataset_collection_id, url=url, params=params)
 
-    def download_dataset_collection(self, dataset_collection_id: str, file_path: str) -> dict:
+    def download_dataset_collection(self, dataset_collection_id: str, file_path: str) -> Dict[str, Any]:
         """
         Download a history dataset collection as an archive.
 
@@ -129,33 +141,35 @@ class DatasetCollectionClient(Client):
         .. note::
           This method downloads a ``zip`` archive for Galaxy 21.01 and later.
           For earlier versions of Galaxy this method downloads a ``tgz`` archive.
-          This method is only supported by Galaxy 18.01 or later.
+          This method works only on Galaxy 18.01 or later.
         """
-        url = self._make_url(module_id=dataset_collection_id) + '/download'
+        url = self._make_url(module_id=dataset_collection_id) + "/download"
         r = self.gi.make_get_request(url, stream=True)
         r.raise_for_status()
 
-        archive_type = 'zip' if self.gi.config.get_version()['version_major'] >= '21.01' else 'tgz'
+        archive_type = "zip" if self.gi.config.get_version()["version_major"] >= "21.01" else "tgz"
 
-        with open(file_path, 'wb') as fp:
+        with open(file_path, "wb") as fp:
             for chunk in r.iter_content(chunk_size=CHUNK_SIZE):
                 if chunk:
                     fp.write(chunk)
 
-        return {
-            'file_path': file_path,
-            'archive_type': archive_type
-        }
+        return {"file_path": file_path, "archive_type": archive_type}
 
-    def wait_for_dataset_collection(self, dataset_collection_id: str, maxwait: float = 12000,
-                                    interval: float = 3, proportion_complete: float = 1.0,
-                                    check: bool = True) -> dict:
+    def wait_for_dataset_collection(
+        self,
+        dataset_collection_id: str,
+        maxwait: float = 12000,
+        interval: float = 3,
+        proportion_complete: float = 1.0,
+        check: bool = True,
+    ) -> Dict[str, Any]:
         """
         Wait until all or a specified proportion of elements of a dataset
         collection are in a terminal state.
 
         :type dataset_collection_id: str
-        :param dataset_id: dataset collection ID
+        :param dataset_collection_id: dataset collection ID
 
         :type maxwait: float
         :param maxwait: Total time (in seconds) to wait for the dataset
@@ -189,21 +203,27 @@ class DatasetCollectionClient(Client):
 
         time_left = maxwait
         while True:
-            dataset_collection = self.gi.dataset_collections.show_dataset_collection(dataset_collection_id)
-            states = [elem['object']['state'] for elem in dataset_collection['elements']]
+            dataset_collection = self.show_dataset_collection(dataset_collection_id)
+            states = [elem["object"]["state"] for elem in dataset_collection["elements"]]
             terminal_states = [state for state in states if state in TERMINAL_STATES]
-            if set(terminal_states) not in [{'ok'}, set()]:
-                raise Exception(f"Dataset collection {dataset_collection_id} contains elements in the "
-                                f"following non-ok terminal states: {', '.join(set(terminal_states) - {'ok'})}")
+            if set(terminal_states) not in [{"ok"}, set()]:
+                raise Exception(
+                    f"Dataset collection {dataset_collection_id} contains elements in the "
+                    f"following non-ok terminal states: {', '.join(set(terminal_states) - {'ok'})}"
+                )
             proportion = len(terminal_states) / len(states)
             if proportion >= proportion_complete:
                 return dataset_collection
             if time_left > 0:
-                log.info(f"The dataset collection {dataset_collection_id} has {len(terminal_states)} out of {len(states)} datasets in a terminal state. Will wait {time_left} more s")
+                log.info(
+                    f"The dataset collection {dataset_collection_id} has {len(terminal_states)} out of {len(states)} datasets in a terminal state. Will wait {time_left} more s"
+                )
                 time.sleep(min(time_left, interval))
                 time_left -= interval
             else:
-                raise DatasetCollectionTimeoutException(f"Less than {proportion_complete * 100}% of datasets in the dataset collection is in a terminal state after {maxwait} s")
+                raise DatasetCollectionTimeoutException(
+                    f"Less than {proportion_complete * 100}% of datasets in the dataset collection is in a terminal state after {maxwait} s"
+                )
 
 
 class DatasetCollectionTimeoutException(TimeoutException):
