@@ -11,27 +11,38 @@ from typing import (
     Dict,
     List,
     Optional,
+    overload,
+    Tuple,
+    TYPE_CHECKING,
     Union,
 )
 from urllib.parse import urljoin
 
+from requests import Response
+from typing_extensions import Literal
+
 import bioblend
 from bioblend import TimeoutException
 from bioblend.galaxy.client import Client
 
+if TYPE_CHECKING:
+    from bioblend.galaxy import GalaxyInstance
+
 log = logging.getLogger(__name__)
 
-TERMINAL_STATES = {'ok', 'empty', 'error', 'discarded', 'failed_metadata'}
+HdaLdda = Literal["hda", "ldda"]
+TERMINAL_STATES = {"ok", "empty", "error", "discarded", "failed_metadata"}
 # Non-terminal states are: 'new', 'upload', 'queued', 'running', 'paused', 'setting_metadata'
 
 
 class DatasetClient(Client):
-    module = 'datasets'
+    gi: "GalaxyInstance"
+    module = "datasets"
 
-    def __init__(self, galaxy_instance):
+    def __init__(self, galaxy_instance: "GalaxyInstance") -> None:
         super().__init__(galaxy_instance)
 
-    def show_dataset(self, dataset_id, deleted=False, hda_ldda='hda'):
+    def show_dataset(self, dataset_id: str, deleted: bool = False, hda_ldda: HdaLdda = "hda") -> Dict[str, Any]:
         """
         Get details about a given dataset. This can be a history or a library dataset.
 
@@ -53,8 +64,64 @@ class DatasetClient(Client):
         )
         return self._get(id=dataset_id, deleted=deleted, params=params)
 
-    def download_dataset(self, dataset_id, file_path=None, use_default_filename=True,
-                         require_ok_state=True, maxwait=12000):
+    def _initiate_download(
+        self, dataset_id: str, stream_content: bool, require_ok_state: bool = True, maxwait: float = 12000
+    ) -> Tuple[Dict[str, Any], str, Response]:
+        dataset = self.wait_for_dataset(dataset_id, maxwait=maxwait, check=False)
+        if not dataset["state"] == "ok":
+            message = f"Dataset state is not 'ok'. Dataset id: {dataset_id}, current state: {dataset['state']}"
+            if require_ok_state:
+                raise DatasetStateException(message)
+            else:
+                warnings.warn(message, DatasetStateWarning)
+
+        file_ext = dataset.get("file_ext")
+        # Resort to 'data' when Galaxy returns an empty or temporary extension
+        if not file_ext or file_ext == "auto" or file_ext == "_sniff_":
+            file_ext = "data"
+        # The preferred download URL is
+        # '/api/histories/<history_id>/contents/<dataset_id>/display?to_ext=<dataset_ext>'
+        # since the old URL:
+        # '/dataset/<dataset_id>/display?to_ext=<dataset_ext>'
+        # does not work when using REMOTE_USER with access disabled to
+        # everything but /api without auth
+        download_url = dataset["download_url"] + "?to_ext=" + file_ext
+        url = urljoin(self.gi.base_url, download_url)
+
+        r = self.gi.make_get_request(url, stream=stream_content)
+        r.raise_for_status()
+        return dataset, file_ext, r
+
+    @overload
+    def download_dataset(
+        self,
+        dataset_id: str,
+        file_path: None = None,
+        use_default_filename: bool = True,
+        require_ok_state: bool = True,
+        maxwait: float = 12000,
+    ) -> bytes:
+        ...
+
+    @overload
+    def download_dataset(
+        self,
+        dataset_id: str,
+        file_path: str,
+        use_default_filename: bool = True,
+        require_ok_state: bool = True,
+        maxwait: float = 12000,
+    ) -> str:
+        ...
+
+    def download_dataset(
+        self,
+        dataset_id: str,
+        file_path: Optional[str] = None,
+        use_default_filename: bool = True,
+        require_ok_state: bool = True,
+        maxwait: float = 12000,
+    ) -> Union[bytes, str]:
         """
         Download a dataset to file or in memory. If the dataset state is not
         'ok', a ``DatasetStateException`` will be thrown, unless ``require_ok_state=False``.
@@ -85,49 +152,31 @@ class DatasetClient(Client):
           become terminal. If the dataset state is not terminal within this
           time, a ``DatasetTimeoutException`` will be thrown.
 
-        :rtype: dict
-        :return: If a ``file_path`` argument is not provided, returns a dict containing the file content.
-                 Otherwise returns nothing.
+        :rtype: bytes or str
+        :return: If a ``file_path`` argument is not provided, returns the file
+          content. Otherwise returns the local path of the downloaded file.
         """
-        dataset = self.wait_for_dataset(dataset_id, maxwait=maxwait, check=False)
-        if not dataset['state'] == 'ok':
-            message = f"Dataset state is not 'ok'. Dataset id: {dataset_id}, current state: {dataset['state']}"
-            if require_ok_state:
-                raise DatasetStateException(message)
-            else:
-                warnings.warn(message, DatasetStateWarning)
-
-        file_ext = dataset.get('file_ext')
-        # Resort to 'data' when Galaxy returns an empty or temporary extension
-        if not file_ext or file_ext == 'auto' or file_ext == '_sniff_':
-            file_ext = 'data'
-        # The preferred download URL is
-        # '/api/histories/<history_id>/contents/<dataset_id>/display?to_ext=<dataset_ext>'
-        # since the old URL:
-        # '/dataset/<dataset_id>/display/to_ext=<dataset_ext>'
-        # does not work when using REMOTE_USER with access disabled to
-        # everything but /api without auth
-        download_url = dataset['download_url'] + '?to_ext=' + file_ext
-        url = urljoin(self.gi.base_url, download_url)
-
-        stream_content = file_path is not None
-        r = self.gi.make_get_request(url, stream=stream_content)
-        r.raise_for_status()
-
+        dataset, file_ext, r = self._initiate_download(
+            dataset_id, stream_content=file_path is not None, require_ok_state=require_ok_state, maxwait=maxwait
+        )
         if file_path is None:
-            if 'content-length' in r.headers and len(r.content) != int(r.headers['content-length']):
-                log.warning("Transferred content size does not match content-length header (%s != %s)", len(r.content), r.headers['content-length'])
+            if "content-length" in r.headers and len(r.content) != int(r.headers["content-length"]):
+                log.warning(
+                    "Transferred content size does not match content-length header (%s != %s)",
+                    len(r.content),
+                    r.headers["content-length"],
+                )
             return r.content
         else:
             if use_default_filename:
                 # Build a useable filename
-                filename = dataset['name'] + '.' + file_ext
+                filename = dataset["name"] + "." + file_ext
                 # Now try to get a better filename from the response headers
                 # We expect tokens 'filename' '=' to be followed by the quoted filename
-                if 'content-disposition' in r.headers:
-                    tokens = list(shlex.shlex(r.headers['content-disposition'], posix=True))
+                if "content-disposition" in r.headers:
+                    tokens = list(shlex.shlex(r.headers["content-disposition"], posix=True))
                     try:
-                        header_filepath = tokens[tokens.index('filename') + 2]
+                        header_filepath = tokens[tokens.index("filename") + 2]
                         filename = os.path.basename(header_filepath)
                     except (ValueError, IndexError):
                         pass
@@ -135,7 +184,7 @@ class DatasetClient(Client):
             else:
                 file_local_path = file_path
 
-            with open(file_local_path, 'wb') as fp:
+            with open(file_local_path, "wb") as fp:
                 for chunk in r.iter_content(chunk_size=bioblend.CHUNK_SIZE):
                     if chunk:
                         fp.write(chunk)
@@ -156,12 +205,12 @@ class DatasetClient(Client):
         tool_id: Optional[str] = None,
         tag: Optional[str] = None,
         history_id: Optional[str] = None,
-        create_time_min: str = None,
-        create_time_max: str = None,
-        update_time_min: str = None,
-        update_time_max: str = None,
-        order: str = 'create_time-dsc',
-    ) -> List[dict]:
+        create_time_min: Optional[str] = None,
+        create_time_max: Optional[str] = None,
+        update_time_min: Optional[str] = None,
+        update_time_max: Optional[str] = None,
+        order: str = "create_time-dsc",
+    ) -> List[Dict[str, Any]]:
         """
         Get the latest datasets, or select another subset by specifying optional
         arguments for filtering (e.g. a history ID).
@@ -234,70 +283,70 @@ class DatasetClient(Client):
         :param: A list of datasets
         """
         params: Dict[str, Any] = {
-            'limit': limit,
-            'offset': offset,
-            'order': order,
+            "limit": limit,
+            "offset": offset,
+            "order": order,
         }
         if history_id:
-            params['history_id'] = history_id
+            params["history_id"] = history_id
 
         q: List[str] = []
-        qv: List[Any] = []
+        qv = []
 
         if name:
-            q.append('name')
+            q.append("name")
             qv.append(name)
         if state:
             op, val = self._param_to_filter(state)
-            q.append(f'state-{op}')
+            q.append(f"state-{op}")
             qv.append(val)
         if extension:
             op, val = self._param_to_filter(extension)
-            q.append(f'extension-{op}')
+            q.append(f"extension-{op}")
             qv.append(val)
         if visible is not None:
-            q.append('visible')
+            q.append("visible")
             qv.append(str(visible))
         if deleted is not None:
-            q.append('deleted')
+            q.append("deleted")
             qv.append(str(deleted))
         if purged is not None:
-            q.append('purged')
+            q.append("purged")
             qv.append(str(purged))
         if tool_id is not None:
-            q.append('tool_id')
+            q.append("tool_id")
             qv.append(str(tool_id))
         if tag is not None:
-            q.append('tag')
+            q.append("tag")
             qv.append(str(tag))
         if create_time_min:
-            q.append('create_time-ge')
+            q.append("create_time-ge")
             qv.append(create_time_min)
         if create_time_max:
-            q.append('create_time-le')
+            q.append("create_time-le")
             qv.append(create_time_max)
         if update_time_min:
-            q.append('update_time-ge')
+            q.append("update_time-ge")
             qv.append(update_time_min)
         if update_time_max:
-            q.append('update_time-le')
+            q.append("update_time-le")
             qv.append(update_time_max)
 
-        params['q'] = q
-        params['qv'] = qv
+        params["q"] = q
+        params["qv"] = qv
 
         return self._get(params=params)
 
-    def _param_to_filter(self, param):
-        if type(param) is str:
-            return 'eq', param
-        if type(param) is list:
+    def _param_to_filter(self, param: Union[str, List[str]]) -> Tuple[str, str]:
+        if isinstance(param, str):
+            return "eq", param
+        if isinstance(param, list):
             if len(param) == 1:
-                return 'eq', param.pop()
-            return 'in', ','.join(param)
+                return "eq", param.pop()
+            return "in", ",".join(param)
         raise Exception("Filter param is not of type ``str`` or ``list``")
 
-    def publish_dataset(self, dataset_id: str, published: bool = False):
+    def publish_dataset(self, dataset_id: str, published: bool = False) -> Dict[str, Any]:
         """
         Make a dataset publicly available or private. For more fine-grained control (assigning different
         permissions to specific roles), use the ``update_permissions()`` method.
@@ -309,19 +358,22 @@ class DatasetClient(Client):
         :param published: Whether to make the dataset published (``True``) or private (``False``).
 
         :rtype: dict
-        :return: Current roles for all available permission types.
+        :return: Details of the updated dataset
 
         .. note::
-          This method can only be used with Galaxy ``release_19.05`` or later.
+          This method works only on Galaxy 19.05 or later.
         """
-        payload: Dict[str, Any] = {
-            'action': 'remove_restrictions' if published else 'make_private'
-        }
-        url = self._make_url(dataset_id) + '/permissions'
-        self.gi.datasets._put(url=url, payload=payload)
+        payload: Dict[str, Any] = {"action": "remove_restrictions" if published else "make_private"}
+        url = self._make_url(dataset_id) + "/permissions"
+        return self.gi.datasets._put(url=url, payload=payload)
 
-    def update_permissions(self, dataset_id: str, access_ids: Optional[list] = None,
-                           manage_ids: Optional[list] = None, modify_ids: Optional[list] = None):
+    def update_permissions(
+        self,
+        dataset_id: str,
+        access_ids: Optional[list] = None,
+        manage_ids: Optional[list] = None,
+        modify_ids: Optional[list] = None,
+    ) -> dict:
         """
         Set access, manage or modify permissions for a dataset to a list of roles.
 
@@ -341,21 +393,21 @@ class DatasetClient(Client):
         :return: Current roles for all available permission types.
 
         .. note::
-          This method can only be used with Galaxy ``release_19.05`` or later.
+          This method works only on Galaxy 19.05 or later.
         """
-        payload: Dict[str, Any] = {
-            'action': 'set_permissions'
-        }
+        payload: Dict[str, Any] = {"action": "set_permissions"}
         if access_ids:
-            payload['access'] = access_ids
+            payload["access"] = access_ids
         if manage_ids:
-            payload['manage'] = manage_ids
+            payload["manage"] = manage_ids
         if modify_ids:
-            payload['modify'] = modify_ids
-        url = self._make_url(dataset_id) + '/permissions'
-        self.gi.datasets._put(url=url, payload=payload)
+            payload["modify"] = modify_ids
+        url = self._make_url(dataset_id) + "/permissions"
+        return self.gi.datasets._put(url=url, payload=payload)
 
-    def wait_for_dataset(self, dataset_id, maxwait=12000, interval=3, check=True):
+    def wait_for_dataset(
+        self, dataset_id: str, maxwait: float = 12000, interval: float = 3, check: bool = True
+    ) -> Dict[str, Any]:
         """
         Wait until a dataset is in a terminal state.
 
@@ -382,9 +434,9 @@ class DatasetClient(Client):
         time_left = maxwait
         while True:
             dataset = self.show_dataset(dataset_id)
-            state = dataset['state']
+            state = dataset["state"]
             if state in TERMINAL_STATES:
-                if check and state != 'ok':
+                if check and state != "ok":
                     raise Exception(f"Dataset {dataset_id} is in terminal state {state}")
                 return dataset
             if time_left > 0:
@@ -392,7 +444,9 @@ class DatasetClient(Client):
                 time.sleep(min(time_left, interval))
                 time_left -= interval
             else:
-                raise DatasetTimeoutException(f"Dataset {dataset_id} is still in non-terminal state {state} after {maxwait} s")
+                raise DatasetTimeoutException(
+                    f"Dataset {dataset_id} is still in non-terminal state {state} after {maxwait} s"
+                )
 
 
 class DatasetStateException(Exception):
