@@ -5,25 +5,42 @@ import logging
 import re
 import sys
 import time
-import warnings
+import typing
 import webbrowser
+from typing import (
+    Any,
+    Dict,
+    IO,
+    List,
+    Optional,
+    overload,
+    Pattern,
+    Union,
+)
 from urllib.parse import urljoin
 
+from typing_extensions import Literal
+
 import bioblend
 from bioblend import ConnectionError
 from bioblend.galaxy.client import Client
+from bioblend.galaxy.dataset_collections import CollectionDescription
 from bioblend.util import attach_file
 
+if typing.TYPE_CHECKING:
+    from bioblend.galaxy import GalaxyInstance
+
 log = logging.getLogger(__name__)
 
 
 class HistoryClient(Client):
-    module = 'histories'
+    gi: "GalaxyInstance"
+    module = "histories"
 
-    def __init__(self, galaxy_instance):
+    def __init__(self, galaxy_instance: "GalaxyInstance") -> None:
         super().__init__(galaxy_instance)
 
-    def create_history(self, name=None):
+    def create_history(self, name: Optional[str] = None) -> Dict[str, Any]:
         """
         Create a new history, optionally setting the ``name``.
 
@@ -35,67 +52,84 @@ class HistoryClient(Client):
         """
         payload = {}
         if name is not None:
-            payload['name'] = name
+            payload["name"] = name
         return self._post(payload)
 
-    def import_history(self, file_path=None, url=None):
+    def import_history(self, file_path: Optional[str] = None, url: Optional[str] = None) -> Dict[str, Any]:
         """
         Import a history from an archive on disk or a URL.
 
         :type file_path: str
         :param file_path: Path to exported history archive on disk.
+
         :type url: str
         :param url: URL for an exported history archive
+
+        :rtype: dict
+        :return: Dictionary containing information about the imported history
         """
         if file_path:
             archive_file = attach_file(file_path)
-            payload = dict(archive_source='', archive_file=archive_file, archive_type="file")
+            payload: Dict[str, Any] = {
+                "archive_source": "",
+                "archive_file": archive_file,
+                "archive_type": "file",
+            }
         else:
-            payload = dict(archive_source=url, archive_type='url')
+            payload = {
+                "archive_source": url,
+                "archive_type": "url",
+            }
 
         return self._post(payload=payload, files_attached=file_path is not None)
 
-    def _get_histories(self, history_id=None, name=None, deleted=False, filter_user_published=None, get_all_published=False, slug=None):
+    def _get_histories(
+        self,
+        name: Optional[str] = None,
+        deleted: bool = False,
+        filter_user_published: Optional[bool] = None,
+        get_all_published: bool = False,
+        slug: Optional[str] = None,
+        all: Optional[bool] = False,
+    ) -> List[Dict[str, Any]]:
         """
         Hidden method to be used by both get_histories() and get_published_histories()
         """
-        if history_id is not None and name is not None:
-            raise ValueError('Provide only one argument between name or history_id, but not both')
         assert not (filter_user_published is not None and get_all_published)
 
-        params = {}
+        params: Dict[str, Any] = {}
         if deleted:
-            params.setdefault('q', []).append('deleted')
-            params.setdefault('qv', []).append(deleted)
+            params.setdefault("q", []).append("deleted")
+            params.setdefault("qv", []).append(deleted)
         if filter_user_published is not None:
-            params.setdefault('q', []).append('published')
-            params.setdefault('qv', []).append(filter_user_published)
+            params.setdefault("q", []).append("published")
+            params.setdefault("qv", []).append(filter_user_published)
         if slug is not None:
-            params.setdefault('q', []).append('slug')
-            params.setdefault('qv', []).append(slug)
+            params.setdefault("q", []).append("slug")
+            params.setdefault("qv", []).append(slug)
+        if all:
+            params["all"] = True
 
-        url = '/'.join((self._make_url(), 'published')) if get_all_published else None
+        url = "/".join((self._make_url(), "published")) if get_all_published else None
         histories = self._get(url=url, params=params)
 
-        if history_id is not None:
-            history = next((_ for _ in histories if _['id'] == history_id), None)
-            histories = [history] if history is not None else []
-        elif name is not None:
-            histories = [_ for _ in histories if _['name'] == name]
+        if name is not None:
+            histories = [_ for _ in histories if _["name"] == name]
         return histories
 
-    def get_histories(self, history_id=None, name=None, deleted=False, published=None, slug=None):
+    def get_histories(
+        self,
+        history_id: Optional[str] = None,
+        name: Optional[str] = None,
+        deleted: bool = False,
+        published: Optional[bool] = None,
+        slug: Optional[str] = None,
+        all: Optional[bool] = False,
+    ) -> List[Dict[str, Any]]:
         """
         Get all histories, or select a subset by specifying optional arguments
         for filtering (e.g. a history name).
 
-        :type history_id: str
-        :param history_id: Encoded history ID to filter on
-
-          .. deprecated:: 0.15.0
-             To get details of a history for which you know the ID, use the much
-             more efficient :meth:`show_history` instead.
-
         :type name: str
         :param name: History name to filter on.
 
@@ -113,17 +147,29 @@ class HistoryClient(Client):
         :type slug: str
         :param slug: History slug to filter on
 
+        :type all: bool
+        :param all: Whether to include histories from other users. This
+          parameter works only on Galaxy 20.01 or later and can be specified
+          only if the user is a Galaxy admin.
+
         :rtype: list
         :return: List of history dicts.
+
+        .. versionchanged:: 0.17.0
+            Using the deprecated ``history_id`` parameter now raises a
+            ``ValueError`` exception.
         """
         if history_id is not None:
-            warnings.warn(
-                'The history_id parameter is deprecated, use the show_history() method to view details of a history for which you know the ID.',
-                category=FutureWarning
+            raise ValueError(
+                "The history_id parameter has been removed, use the show_history() method to view details of a history for which you know the ID.",
             )
-        return self._get_histories(history_id=history_id, name=name, deleted=deleted, filter_user_published=published, get_all_published=False, slug=slug)
+        return self._get_histories(
+            name=name, deleted=deleted, filter_user_published=published, get_all_published=False, slug=slug, all=all
+        )
 
-    def get_published_histories(self, name=None, deleted=False, slug=None):
+    def get_published_histories(
+        self, name: Optional[str] = None, deleted: bool = False, slug: Optional[str] = None
+    ) -> List[Dict[str, Any]]:
         """
         Get all published histories (by any user), or select a subset by
         specifying optional arguments for filtering (e.g. a history name).
@@ -141,9 +187,52 @@ class HistoryClient(Client):
         :rtype: list
         :return: List of history dicts.
         """
-        return self._get_histories(name=name, deleted=deleted, filter_user_published=None, get_all_published=True, slug=slug)
+        return self._get_histories(
+            name=name, deleted=deleted, filter_user_published=None, get_all_published=True, slug=slug
+        )
 
-    def show_history(self, history_id, contents=False, deleted=None, visible=None, details=None, types=None):
+    @overload
+    def show_history(
+        self,
+        history_id: str,
+        contents: Literal[False] = False,
+    ) -> Dict[str, Any]:
+        ...
+
+    @overload
+    def show_history(
+        self,
+        history_id: str,
+        contents: Literal[True],
+        deleted: Optional[bool] = None,
+        visible: Optional[bool] = None,
+        details: Optional[str] = None,
+        types: Optional[List[str]] = None,
+    ) -> List[Dict[str, Any]]:
+        ...
+
+    # Fallback in case the caller provides a regular bool as contents
+    @overload
+    def show_history(
+        self,
+        history_id: str,
+        contents: bool = False,
+        deleted: Optional[bool] = None,
+        visible: Optional[bool] = None,
+        details: Optional[str] = None,
+        types: Optional[List[str]] = None,
+    ) -> Union[Dict[str, Any], List[Dict[str, Any]]]:
+        pass
+
+    def show_history(
+        self,
+        history_id: str,
+        contents: bool = False,
+        deleted: Optional[bool] = None,
+        visible: Optional[bool] = None,
+        details: Optional[str] = None,
+        types: Optional[List[str]] = None,
+    ) -> Union[Dict[str, Any], List[Dict[str, Any]]]:
         """
         Get details of a given history. By default, just get the history meta
         information.
@@ -187,19 +276,19 @@ class HistoryClient(Client):
             more extensive functionality for filtering and ordering the results.
 
         """
-        params = {}
+        params: Dict[str, Union[bool, list, str]] = {}
         if contents:
             if details:
-                params['details'] = details
+                params["details"] = details
             if deleted is not None:
-                params['deleted'] = deleted
+                params["deleted"] = deleted
             if visible is not None:
-                params['visible'] = visible
+                params["visible"] = visible
             if types is not None:
-                params['types'] = types
+                params["types"] = types
         return self._get(id=history_id, contents=contents, params=params)
 
-    def delete_dataset(self, history_id, dataset_id, purge=False):
+    def delete_dataset(self, history_id: str, dataset_id: str, purge: bool = False) -> None:
         """
         Mark corresponding dataset as deleted.
 
@@ -216,17 +305,17 @@ class HistoryClient(Client):
         :return: None
 
         .. note::
-            For the purge option to work, the Galaxy instance must have the
-            ``allow_user_dataset_purge`` option set to ``true`` in the
-            ``config/galaxy.yml`` configuration file.
+          The ``purge`` option works only if the Galaxy instance has the
+          ``allow_user_dataset_purge`` option set to ``true`` in the
+          ``config/galaxy.yml`` configuration file.
         """
-        url = '/'.join((self._make_url(history_id, contents=True), dataset_id))
+        url = "/".join((self._make_url(history_id, contents=True), dataset_id))
         payload = {}
         if purge is True:
-            payload['purge'] = purge
+            payload["purge"] = purge
         self._delete(payload=payload, url=url)
 
-    def delete_dataset_collection(self, history_id, dataset_collection_id):
+    def delete_dataset_collection(self, history_id: str, dataset_collection_id: str) -> None:
         """
         Mark corresponding dataset collection as deleted.
 
@@ -239,10 +328,10 @@ class HistoryClient(Client):
         :rtype: None
         :return: None
         """
-        url = '/'.join((self._make_url(history_id, contents=True), 'dataset_collections', dataset_collection_id))
+        url = "/".join((self._make_url(history_id, contents=True), "dataset_collections", dataset_collection_id))
         self._delete(url=url)
 
-    def show_dataset(self, history_id, dataset_id):
+    def show_dataset(self, history_id: str, dataset_id: str) -> Dict[str, Any]:
         """
         Get details about a given history dataset.
 
@@ -255,10 +344,10 @@ class HistoryClient(Client):
         :rtype: dict
         :return: Information about the dataset
         """
-        url = '/'.join((self._make_url(history_id, contents=True), dataset_id))
+        url = "/".join((self._make_url(history_id, contents=True), dataset_id))
         return self._get(url=url)
 
-    def show_dataset_collection(self, history_id, dataset_collection_id):
+    def show_dataset_collection(self, history_id: str, dataset_collection_id: str) -> Dict[str, Any]:
         """
         Get details about a given history dataset collection.
 
@@ -271,10 +360,12 @@ class HistoryClient(Client):
         :rtype: dict
         :return: Information about the dataset collection
         """
-        url = '/'.join((self._make_url(history_id, contents=True), 'dataset_collections', dataset_collection_id))
+        url = "/".join((self._make_url(history_id, contents=True), "dataset_collections", dataset_collection_id))
         return self._get(url=url)
 
-    def show_matching_datasets(self, history_id, name_filter=None):
+    def show_matching_datasets(
+        self, history_id: str, name_filter: Optional[Union[str, Pattern[str]]] = None
+    ) -> List[Dict[str, Any]]:
         """
         Get dataset details for matching datasets within a history.
 
@@ -291,12 +382,14 @@ class HistoryClient(Client):
         :return: List of dictionaries
         """
         if isinstance(name_filter, str):
-            name_filter = re.compile(name_filter + '$')
-        return [self.show_dataset(history_id, h['id'])
-                for h in self.show_history(history_id, contents=True)
-                if name_filter is None or name_filter.match(h['name'])]
+            name_filter = re.compile(name_filter + "$")
+        return [
+            self.show_dataset(history_id, h["id"])
+            for h in self.show_history(history_id, contents=True)
+            if name_filter is None or name_filter.match(h["name"])
+        ]
 
-    def show_dataset_provenance(self, history_id, dataset_id, follow=False):
+    def show_dataset_provenance(self, history_id: str, dataset_id: str, follow: bool = False) -> Dict[str, Any]:
         """
         Get details related to how dataset was created (``id``, ``job_id``,
         ``tool_id``, ``stdout``, ``stderr``, ``parameters``, ``inputs``,
@@ -331,10 +424,11 @@ class HistoryClient(Client):
              'tool_id': 'toolshed.g2.bx.psu.edu/repos/ziru-zhou/macs2/modencode_peakcalling_macs2/2.0.10.2',
              'uuid': '5c0c43f5-8d93-44bd-939d-305e82f213c6'}
         """
-        url = '/'.join((self._make_url(history_id, contents=True), dataset_id, 'provenance'))
-        return self._get(url=url)
+        url = "/".join((self._make_url(history_id, contents=True), dataset_id, "provenance"))
+        params = {"follow": follow}
+        return self._get(url=url, params=params)
 
-    def update_history(self, history_id, **kwds):
+    def update_history(self, history_id: str, **kwargs: Any) -> Dict[str, Any]:
         """
         Update history metadata information. Some of the attributes that can be
         modified are documented below.
@@ -369,9 +463,9 @@ class HistoryClient(Client):
         .. versionchanged:: 0.8.0
             Changed the return value from the status code (type int) to a dict.
         """
-        return self._put(payload=kwds, id=history_id)
+        return self._put(payload=kwargs, id=history_id)
 
-    def update_dataset(self, history_id, dataset_id, **kwds):
+    def update_dataset(self, history_id: str, dataset_id: str, **kwargs: Any) -> Dict[str, Any]:
         """
         Update history dataset metadata. Some of the attributes that can be
         modified are documented below.
@@ -410,10 +504,10 @@ class HistoryClient(Client):
         .. versionchanged:: 0.8.0
             Changed the return value from the status code (type int) to a dict.
         """
-        url = '/'.join((self._make_url(history_id, contents=True), dataset_id))
-        return self._put(payload=kwds, url=url)
+        url = "/".join((self._make_url(history_id, contents=True), dataset_id))
+        return self._put(payload=kwargs, url=url)
 
-    def update_dataset_collection(self, history_id, dataset_collection_id, **kwds):
+    def update_dataset_collection(self, history_id: str, dataset_collection_id: str, **kwargs: Any) -> Dict[str, Any]:
         """
         Update history dataset collection metadata. Some of the attributes that
         can be modified are documented below.
@@ -440,10 +534,10 @@ class HistoryClient(Client):
         .. versionchanged:: 0.8.0
             Changed the return value from the status code (type int) to a dict.
         """
-        url = '/'.join((self._make_url(history_id, contents=True), 'dataset_collections', dataset_collection_id))
-        return self._put(payload=kwds, url=url)
+        url = "/".join((self._make_url(history_id, contents=True), "dataset_collections", dataset_collection_id))
+        return self._put(payload=kwargs, url=url)
 
-    def create_history_tag(self, history_id, tag):
+    def create_history_tag(self, history_id: str, tag: str) -> Dict[str, Any]:
         """
         Create history tag
 
@@ -463,11 +557,11 @@ class HistoryClient(Client):
              'user_value': None}
         """
         # empty payload since we are adding the new tag using the url
-        payload = {}
-        url = '/'.join((self._make_url(history_id), 'tags', tag))
+        payload: Dict[str, Any] = {}
+        url = "/".join((self._make_url(history_id), "tags", tag))
         return self._post(payload, url=url)
 
-    def upload_dataset_from_library(self, history_id, lib_dataset_id):
+    def upload_dataset_from_library(self, history_id: str, lib_dataset_id: str) -> Dict[str, Any]:
         """
         Upload a dataset into the history from a library. Requires the
         library dataset ID, which can be obtained from the library
@@ -483,13 +577,15 @@ class HistoryClient(Client):
         :return: Information about the newly created HDA
         """
         payload = {
-            'content': lib_dataset_id,
-            'source': 'library',
-            'from_ld_id': lib_dataset_id,  # compatibility with old API
+            "content": lib_dataset_id,
+            "source": "library",
+            "from_ld_id": lib_dataset_id,  # compatibility with old API
         }
         return self._post(payload, id=history_id, contents=True)
 
-    def create_dataset_collection(self, history_id, collection_description):
+    def create_dataset_collection(
+        self, history_id: str, collection_description: Union["CollectionDescription", Dict[str, Any]]
+    ) -> Dict[str, Any]:
         """
         Create a new dataset collection
 
@@ -512,19 +608,20 @@ class HistoryClient(Client):
         :rtype: dict
         :return: Information about the new HDCA
         """
-        try:
-            collection_description = collection_description.to_dict()
-        except AttributeError:
-            pass
+        if isinstance(collection_description, CollectionDescription):
+            collection_description_dict = collection_description.to_dict()
+        else:
+            collection_description_dict = collection_description
+
         payload = dict(
-            name=collection_description["name"],
+            name=collection_description_dict["name"],
             type="dataset_collection",
-            collection_type=collection_description["collection_type"],
-            element_identifiers=collection_description["element_identifiers"],
+            collection_type=collection_description_dict["collection_type"],
+            element_identifiers=collection_description_dict["element_identifiers"],
         )
         return self._post(payload, id=history_id, contents=True)
 
-    def delete_history(self, history_id, purge=False):
+    def delete_history(self, history_id: str, purge: bool = False) -> Dict[str, Any]:
         """
         Delete a history.
 
@@ -541,16 +638,16 @@ class HistoryClient(Client):
                  purged).
 
         .. note::
-          For the purge option to work, the Galaxy instance must have the
+          The ``purge`` option works only if the Galaxy instance has the
           ``allow_user_dataset_purge`` option set to ``true`` in the
           ``config/galaxy.yml`` configuration file.
         """
         payload = {}
         if purge is True:
-            payload['purge'] = purge
+            payload["purge"] = purge
         return self._delete(payload=payload, id=history_id)
 
-    def undelete_history(self, history_id):
+    def undelete_history(self, history_id: str) -> str:
         """
         Undelete a history
 
@@ -560,10 +657,10 @@ class HistoryClient(Client):
         :rtype: str
         :return: 'OK' if it was deleted
         """
-        url = self._make_url(history_id, deleted=True) + '/undelete'
-        return self._post(payload={}, url=url)
+        url = self._make_url(history_id, deleted=True) + "/undelete"
+        return self._post(url=url)
 
-    def get_status(self, history_id):
+    def get_status(self, history_id: str) -> Dict[str, Any]:
         """
         Returns the state of this history
 
@@ -576,30 +673,40 @@ class HistoryClient(Client):
             'state_details' = Contains individual statistics for various dataset states.
             'percent_complete' = The overall number of datasets processed to completion.
         """
-        state = {}
         history = self.show_history(history_id)
-        state['state'] = history['state']
-        if history.get('state_details') is not None:
-            state['state_details'] = history['state_details']
-            total_complete = sum(history['state_details'].values())
+
+        state: Dict[str, Any] = {
+            "state": history["state"],
+        }
+
+        if history.get("state_details") is not None:
+            state["state_details"] = history["state_details"]
+            total_complete = sum(history["state_details"].values())
             if total_complete > 0:
-                state['percent_complete'] = 100 * history['state_details']['ok'] / total_complete
+                state["percent_complete"] = 100 * history["state_details"]["ok"] / total_complete
             else:
-                state['percent_complete'] = 0
+                state["percent_complete"] = 0
         return state
 
-    def get_most_recently_used_history(self):
+    def get_most_recently_used_history(self) -> Dict[str, Any]:
         """
         Returns the current user's most recently used history (not deleted).
 
         :rtype: dict
         :return: History representation
         """
-        url = self._make_url() + '/most_recently_used'
+        url = self._make_url() + "/most_recently_used"
         return self._get(url=url)
 
-    def export_history(self, history_id, gzip=True, include_hidden=False,
-                       include_deleted=False, wait=False, maxwait=None):
+    def export_history(
+        self,
+        history_id: str,
+        gzip: bool = True,
+        include_hidden: bool = False,
+        include_deleted: bool = False,
+        wait: bool = False,
+        maxwait: Optional[float] = None,
+    ) -> str:
         """
         Start a job to create an export archive for the given history.
 
@@ -637,32 +744,37 @@ class HistoryClient(Client):
             else:
                 maxwait = 0
         params = {
-            'gzip': gzip,
-            'include_hidden': include_hidden,
-            'include_deleted': include_deleted,
+            "gzip": gzip,
+            "include_hidden": include_hidden,
+            "include_deleted": include_deleted,
         }
         url = f"{self._make_url(history_id)}/exports"
         time_left = maxwait
         while True:
             try:
-                r = self._put(payload={}, url=url, params=params)
+                r = self._put(url=url, params=params)
             except ConnectionError as e:
                 if e.status_code == 202:  # export is not ready
                     if time_left > 0:
-                        log.info("Waiting for the export of history %s to complete. Will wait %i more s", history_id, time_left)
+                        log.info(
+                            "Waiting for the export of history %s to complete. Will wait %i more s",
+                            history_id,
+                            time_left,
+                        )
                         time.sleep(1)
                         time_left -= 1
                     else:
-                        return ''
+                        return ""
                 else:
                     raise
             else:
                 break
-        jeha_id = r['download_url'].rsplit('/', 1)[-1]
+        jeha_id = r["download_url"].rsplit("/", 1)[-1]
         return jeha_id
 
-    def download_history(self, history_id, jeha_id, outf,
-                         chunk_size=bioblend.CHUNK_SIZE):
+    def download_history(
+        self, history_id: str, jeha_id: str, outf: IO[bytes], chunk_size: int = bioblend.CHUNK_SIZE
+    ) -> None:
         """
         Download a history export archive.  Use :meth:`export_history`
         to create an export.
@@ -689,7 +801,9 @@ class HistoryClient(Client):
         for chunk in r.iter_content(chunk_size):
             outf.write(chunk)
 
-    def copy_dataset(self, history_id, dataset_id, source='hda'):
+    def copy_dataset(
+        self, history_id: str, dataset_id: str, source: Literal["hda", "library", "library_folder"] = "hda"
+    ) -> Dict[str, Any]:
         """
         Copy a dataset to a history.
 
@@ -707,7 +821,9 @@ class HistoryClient(Client):
         """
         return self.copy_content(history_id, dataset_id, source)
 
-    def copy_content(self, history_id, content_id, source='hda'):
+    def copy_content(
+        self, history_id: str, content_id: str, source: Literal["hda", "hdca", "library", "library_folder"] = "hda"
+    ) -> Dict[str, Any]:
         """
         Copy existing content (e.g. a dataset) to a history.
 
@@ -728,15 +844,15 @@ class HistoryClient(Client):
         """
 
         payload = {
-            'content': content_id,
-            'source': source,
-            'type': 'dataset' if source != 'hdca' else 'dataset_collection',
+            "content": content_id,
+            "source": source,
+            "type": "dataset" if source != "hdca" else "dataset_collection",
         }
 
         url = self._make_url(history_id, contents=True)
         return self._post(payload=payload, url=url)
 
-    def open_history(self, history_id):
+    def open_history(self, history_id: str) -> None:
         """
         Open Galaxy in a new tab of the default web browser and switch to the
         specified history.
@@ -756,3 +872,23 @@ class HistoryClient(Client):
 
         url = urljoin(self.gi.base_url, f"history/switch_to_history?hist_id={history_id}")
         webbrowser.open_new_tab(url)
+
+    def get_extra_files(self, history_id: str, dataset_id: str) -> List[str]:
+        """
+        Get extra files associated with a composite dataset, or an empty list if
+        there are none.
+
+        :type history_id: str
+        :param history_id: history ID
+
+        :type dataset_id: str
+        :param dataset_id: dataset ID
+
+        :rtype: list
+        :return: List of extra files
+
+        .. note::
+          This method works only on Galaxy 19.01 or later.
+        """
+        url = "/".join((self._make_url(history_id, contents=True), dataset_id, "extra_files"))
+        return self._get(url=url)
