@@ -4,29 +4,49 @@ Contains possible interactions with the Galaxy Jobs
 import logging
 import time
 from typing import (
+    Any,
+    Dict,
     List,
     Optional,
+    TYPE_CHECKING,
 )
 
+from typing_extensions import Literal
+
 from bioblend import TimeoutException
 from bioblend.galaxy.client import Client
 
+if TYPE_CHECKING:
+    from bioblend.galaxy import GalaxyInstance
+
 log = logging.getLogger(__name__)
 
-JOB_TERMINAL_STATES = {'deleted', 'error', 'ok'}
+JOB_TERMINAL_STATES = {"deleted", "deleting", "error", "ok"}
 # Job non-terminal states are: 'deleted_new', 'failed', 'new', 'paused',
 # 'queued', 'resubmitted', 'running', 'upload', 'waiting'
 
 
 class JobsClient(Client):
-    module = 'jobs'
+    module = "jobs"
 
-    def __init__(self, galaxy_instance):
+    def __init__(self, galaxy_instance: "GalaxyInstance") -> None:
         super().__init__(galaxy_instance)
 
-    def get_jobs(self, state=None, history_id=None, invocation_id=None, tool_id=None,
-                 workflow_id=None, user_id=None, date_range_min=None, date_range_max=None,
-                 limit=500, offset=0, user_details=False, order_by=None):
+    def get_jobs(
+        self,
+        state: Optional[str] = None,
+        history_id: Optional[str] = None,
+        invocation_id: Optional[str] = None,
+        tool_id: Optional[str] = None,
+        workflow_id: Optional[str] = None,
+        user_id: Optional[str] = None,
+        date_range_min: Optional[str] = None,
+        date_range_max: Optional[str] = None,
+        limit: int = 500,
+        offset: int = 0,
+        user_details: bool = False,
+        order_by: Literal["create_time", "update_time"] = "update_time",
+    ) -> List[Dict[str, Any]]:
         """
         Get all jobs, or select a subset by specifying optional arguments for
         filtering (e.g. a state).
@@ -97,36 +117,33 @@ class JobsClient(Client):
               'update_time': '2014-03-01T16:05:39.558458'}]
 
         .. note::
-          The following options work only on Galaxy 21.05 or later: ``user_id``,
+          The following parameters work only on Galaxy 21.05 or later: ``user_id``,
           ``limit``, ``offset``, ``workflow_id``, ``invocation_id``.
         """
-        params = {
-            'limit': limit,
-            'offset': offset
-        }
+        params: Dict[str, Any] = {"limit": limit, "offset": offset}
         if state:
-            params['state'] = state
+            params["state"] = state
         if history_id:
-            params['history_id'] = history_id
+            params["history_id"] = history_id
         if invocation_id:
-            params['invocation_id'] = invocation_id
+            params["invocation_id"] = invocation_id
         if tool_id:
-            params['tool_id'] = tool_id
+            params["tool_id"] = tool_id
         if workflow_id:
-            params['workflow_id'] = workflow_id
+            params["workflow_id"] = workflow_id
         if user_id:
-            params['user_id'] = user_id
+            params["user_id"] = user_id
         if date_range_min:
-            params['date_range_min'] = date_range_min
+            params["date_range_min"] = date_range_min
         if date_range_max:
-            params['date_range_max'] = date_range_max
+            params["date_range_max"] = date_range_max
         if user_details:
-            params['user_details'] = user_details
+            params["user_details"] = user_details
         if order_by:
-            params['order_by'] = order_by
+            params["order_by"] = order_by
         return self._get(params=params)
 
-    def show_job(self, job_id, full_details=False):
+    def show_job(self, job_id: str, full_details: bool = False) -> Dict[str, Any]:
         """
         Get details of a given job of the current user.
 
@@ -157,11 +174,11 @@ class JobsClient(Client):
         """
         params = {}
         if full_details:
-            params['full'] = full_details
+            params["full"] = full_details
 
         return self._get(id=job_id, params=params)
 
-    def _build_for_rerun(self, job_id):
+    def _build_for_rerun(self, job_id: str) -> Dict[str, Any]:
         """
         Get details of a given job that can be used to rerun the corresponding tool.
 
@@ -172,10 +189,16 @@ class JobsClient(Client):
         :return: A description of the given job, with all parameters required to rerun.
 
         """
-        url = '/'.join((self._make_url(job_id), 'build_for_rerun'))
+        url = "/".join((self._make_url(job_id), "build_for_rerun"))
         return self._get(url=url)
 
-    def rerun_job(self, job_id, remap=False, tool_inputs_update=None, history_id=None):
+    def rerun_job(
+        self,
+        job_id: str,
+        remap: bool = False,
+        tool_inputs_update: Optional[Dict[str, Any]] = None,
+        history_id: Optional[str] = None,
+    ) -> Dict[str, Any]:
         """
         Rerun a job.
 
@@ -207,31 +230,30 @@ class JobsClient(Client):
           This method works only on Galaxy 21.01 or later.
         """
         job_rerun_params = self._build_for_rerun(job_id)
-        job_inputs = job_rerun_params['state_inputs']
+        job_inputs = job_rerun_params["state_inputs"]
 
         if remap:
-            if not job_rerun_params['job_remap']:
-                raise ValueError('remap was set to True, but this job is not remappable.')
-            job_inputs['rerun_remap_job_id'] = job_id
+            if not job_rerun_params["job_remap"]:
+                raise ValueError("remap was set to True, but this job is not remappable.")
+            job_inputs["rerun_remap_job_id"] = job_id
 
-        def update_inputs(inputs, tool_inputs_update):
-            # recursively update inputs with tool_inputs_update
+        def update_inputs(inputs: Dict[str, Any], tool_inputs_update: Dict[str, Any]) -> None:
+            """Recursively update inputs with tool_inputs_update"""
             for input_param, input_value in tool_inputs_update.items():
-                if type(input_value) is dict:
+                if isinstance(input_value, dict):
                     update_inputs(inputs[input_param], input_value)
                 else:
                     inputs[input_param] = input_value
-            return inputs
 
         if tool_inputs_update:
             update_inputs(job_inputs, tool_inputs_update)
 
-        url = '/'.join((self.gi.url, 'tools'))
+        url = "/".join((self.gi.url, "tools"))
         payload = {
-            "history_id": history_id if history_id else job_rerun_params['history_id'],
-            "tool_id": job_rerun_params['id'],
+            "history_id": history_id if history_id else job_rerun_params["history_id"],
+            "tool_id": job_rerun_params["id"],
             "inputs": job_inputs,
-            "input_format": '21.01'
+            "input_format": "21.01",
         }
         return self._post(url=url, payload=payload)
 
@@ -249,9 +271,9 @@ class JobsClient(Client):
 
         .. versionadded:: 0.5.3
         """
-        return self.show_job(job_id).get('state', '')
+        return self.show_job(job_id).get("state", "")
 
-    def search_jobs(self, tool_id: str, inputs: dict, state: Optional[str] = None) -> List[dict]:
+    def search_jobs(self, tool_id: str, inputs: Dict[str, Any], state: Optional[str] = None) -> List[Dict[str, Any]]:
         """
         Return jobs matching input parameters.
 
@@ -280,15 +302,15 @@ class JobsClient(Client):
           This method works only on Galaxy 18.01 or later.
         """
         job_info = {
-            'tool_id': tool_id,
-            'inputs': inputs,
+            "tool_id": tool_id,
+            "inputs": inputs,
         }
         if state:
-            job_info['state'] = state
-        url = self._make_url() + '/search'
+            job_info["state"] = state
+        url = self._make_url() + "/search"
         return self._post(url=url, payload=job_info)
 
-    def get_metrics(self, job_id: str) -> List[dict]:
+    def get_metrics(self, job_id: str) -> List[Dict[str, Any]]:
         """
         Return job metrics for a given job.
 
@@ -305,10 +327,10 @@ class JobsClient(Client):
           ``expose_potentially_sensitive_job_metrics`` option set to ``true`` in
           the ``config/galaxy.yml`` configuration file.
         """
-        url = self._make_url(module_id=job_id) + '/metrics'
+        url = self._make_url(module_id=job_id) + "/metrics"
         return self._get(url=url)
 
-    def cancel_job(self, job_id: str):
+    def cancel_job(self, job_id: str) -> bool:
         """
         Cancel a job, deleting output datasets.
 
@@ -321,7 +343,7 @@ class JobsClient(Client):
         """
         return self._delete(id=job_id)
 
-    def report_error(self, job_id: str, dataset_id: str, message: str, email: str = None) -> dict:
+    def report_error(self, job_id: str, dataset_id: str, message: str, email: Optional[str] = None) -> Dict[str, Any]:
         """
         Report an error for a given job and dataset to the server administrators.
 
@@ -351,10 +373,10 @@ class JobsClient(Client):
         if email is not None:
             payload["email"] = email
 
-        url = self._make_url(module_id=job_id) + '/error'
+        url = self._make_url(module_id=job_id) + "/error"
         return self._post(url=url, payload=payload)
 
-    def get_common_problems(self, job_id: str) -> dict:
+    def get_common_problems(self, job_id: str) -> Dict[str, Any]:
         """
         Query inputs and jobs for common potential problems that might
         have resulted in job failure.
@@ -368,10 +390,10 @@ class JobsClient(Client):
         .. note::
           This method works only on Galaxy 19.05 or later.
         """
-        url = self._make_url(module_id=job_id) + '/common_problems'
+        url = self._make_url(module_id=job_id) + "/common_problems"
         return self._get(url=url)
 
-    def get_inputs(self, job_id: str) -> List[dict]:
+    def get_inputs(self, job_id: str) -> List[Dict[str, Any]]:
         """
         Get dataset inputs used by a job.
 
@@ -381,10 +403,10 @@ class JobsClient(Client):
         :rtype: list of dicts
         :return: Inputs for the given job
         """
-        url = self._make_url(module_id=job_id) + '/inputs'
+        url = self._make_url(module_id=job_id) + "/inputs"
         return self._get(url=url)
 
-    def get_outputs(self, job_id: str) -> List[dict]:
+    def get_outputs(self, job_id: str) -> List[Dict[str, Any]]:
         """
         Get dataset outputs produced by a job.
 
@@ -394,26 +416,26 @@ class JobsClient(Client):
         :rtype: list of dicts
         :return: Outputs of the given job
         """
-        url = self._make_url(module_id=job_id) + '/outputs'
+        url = self._make_url(module_id=job_id) + "/outputs"
         return self._get(url=url)
 
-    def resume_job(self, job_id: str) -> dict:
+    def resume_job(self, job_id: str) -> List[Dict[str, Any]]:
         """
         Resume a job if it is paused.
 
         :type job_id: str
         :param job_id: job ID
 
-        :rtype: dict
-        :return: dict containing output dataset associations
+        :rtype: list of dicts
+        :return: list of dictionaries containing output dataset associations
 
         .. note::
           This method works only on Galaxy 18.09 or later.
         """
-        url = self._make_url(module_id=job_id) + '/resume'
+        url = self._make_url(module_id=job_id) + "/resume"
         return self._put(url=url)
 
-    def get_destination_params(self, job_id: str) -> dict:
+    def get_destination_params(self, job_id: str) -> Dict[str, Any]:
         """
         Get destination parameters for a job, describing
         the environment and location where the job is run.
@@ -428,7 +450,7 @@ class JobsClient(Client):
           This method works only on Galaxy 20.05 or later and if the user is a
           Galaxy admin.
         """
-        url = self._make_url(module_id=job_id) + '/destination_params'
+        url = self._make_url(module_id=job_id) + "/destination_params"
         return self._get(url=url)
 
     def show_job_lock(self) -> bool:
@@ -443,11 +465,11 @@ class JobsClient(Client):
           This method works only on Galaxy 20.05 or later and if the user is a
           Galaxy admin.
         """
-        url = self.gi.url + '/job_lock'
+        url = self.gi.url + "/job_lock"
         response = self._get(url=url)
-        return response['active']
+        return response["active"]
 
-    def update_job_lock(self, active=False) -> bool:
+    def update_job_lock(self, active: bool = False) -> bool:
         """
         Update the job lock status by setting ``active`` to either
         ``True`` or ``False``. If ``True``, all job dispatching will
@@ -461,13 +483,15 @@ class JobsClient(Client):
           Galaxy admin.
         """
         payload = {
-            'active': active,
+            "active": active,
         }
-        url = self.gi.url + '/job_lock'
+        url = self.gi.url + "/job_lock"
         response = self._put(url=url, payload=payload)
-        return response['active']
+        return response["active"]
 
-    def wait_for_job(self, job_id, maxwait=12000, interval=3, check=True):
+    def wait_for_job(
+        self, job_id: str, maxwait: float = 12000, interval: float = 3, check: bool = True
+    ) -> Dict[str, Any]:
         """
         Wait until a job is in a terminal state.
 
@@ -494,9 +518,9 @@ class JobsClient(Client):
         time_left = maxwait
         while True:
             job = self.show_job(job_id)
-            state = job['state']
+            state = job["state"]
             if state in JOB_TERMINAL_STATES:
-                if check and state != 'ok':
+                if check and state != "ok":
                     raise Exception(f"Job {job_id} is in terminal state {state}")
                 return job
             if time_left > 0:
