@@ -6,32 +6,44 @@ via their handles in :class:`~.galaxy_instance.GalaxyInstance`.
 """
 import abc
 import json
-from collections.abc import (
-    Mapping,
-    Sequence,
+from collections.abc import Sequence
+from typing import (
+    Any,
+    cast,
+    Dict,
+    Generic,
+    List,
+    Optional,
+    overload,
+    Type,
+    TYPE_CHECKING,
+    Union,
 )
-from typing import List
+
+from typing_extensions import Literal
 
 import bioblend
+from bioblend.galaxy.datasets import HdaLdda
 from . import wrappers
 
+if TYPE_CHECKING:
+    from .galaxy_instance import GalaxyInstance
+
 
 class ObjClient(abc.ABC):
-
-    def __init__(self, obj_gi):
+    def __init__(self, obj_gi: "GalaxyInstance") -> None:
         self.obj_gi = obj_gi
         self.gi = self.obj_gi.gi
         self.log = bioblend.log
 
     @abc.abstractmethod
-    def get(self, id_) -> wrappers.Wrapper:
+    def get(self, id_: str) -> wrappers.Wrapper:
         """
         Retrieve the object corresponding to the given id.
         """
-        pass
 
     @abc.abstractmethod
-    def get_previews(self) -> list:
+    def get_previews(self, **kwargs: Any) -> list:
         """
         Get a list of object previews.
 
@@ -43,7 +55,6 @@ class ObjClient(abc.ABC):
         :rtype: list
         :return: a list of object previews
         """
-        pass
 
     @abc.abstractmethod
     def list(self) -> list:
@@ -56,61 +67,79 @@ class ObjClient(abc.ABC):
         :rtype: list
         :return: a list of objects
         """
-        pass
 
-    def _select_ids(self, id_=None, name=None):
+    def _select_id(self, id_: Optional[str] = None, name: Optional[str] = None) -> str:
         """
-        Return the id list that corresponds to the given id or name info.
+        Return the id that corresponds to the given id or name info.
         """
         if id_ is None and name is None:
-            self._error('neither id nor name provided', err_type=TypeError)
+            raise ValueError("Neither id nor name provided")
         if id_ is not None and name is not None:
-            self._error('both id and name provided', err_type=TypeError)
+            raise ValueError("Both id and name provided")
         if id_ is None:
-            return [_.id for _ in self.get_previews(name=name)]
+            id_list = [_.id for _ in self.get_previews(name=name)]
+            if len(id_list) > 1:
+                raise ValueError(f"Ambiguous name '{name}'")
+            if not id_list:
+                raise ValueError(f"Name '{name}' not found")
+            return id_list[0]
         else:
-            return [id_]
+            return id_
 
-    def _error(self, msg, err_type=RuntimeError):
-        self.log.error(msg)
-        raise err_type(msg)
-
-    def _get_dict(self, meth_name, reply):
+    def _get_dict(self, meth_name: str, reply: Optional[Union[Dict[str, Any], List[Dict[str, Any]]]]) -> Dict[str, Any]:
         if reply is None:
-            self._error(f"{meth_name}: no reply")
-        elif isinstance(reply, Mapping):
+            raise RuntimeError(f"{meth_name}: no reply")
+        elif isinstance(reply, dict):
             return reply
         try:
             return reply[0]
         except (TypeError, IndexError):
-            self._error(f'{meth_name}: unexpected reply: {reply!r}')
+            raise RuntimeError(f"{meth_name}: unexpected reply: {reply!r}")
 
 
-class ObjDatasetContainerClient(ObjClient):
+class ObjDatasetContainerClient(
+    ObjClient, Generic[wrappers.DatasetContainerSubtype, wrappers.DatasetContainerPreviewSubtype]
+):
+    CONTAINER_TYPE: Type[wrappers.DatasetContainer]
+    CONTAINER_PREVIEW_TYPE: Type[wrappers.DatasetContainerPreview]
 
-    def _get_container(self, id_, ctype):
-        show_fname = f"show_{ctype.__name__.lower()}"
-        gi_client = getattr(self.gi, ctype.API_MODULE)
-        show_f = getattr(gi_client, show_fname)
-        res = show_f(id_)
-        cdict = self._get_dict(show_fname, res)
-        cdict['id'] = id_  # overwrite unencoded id
-        c_infos = show_f(id_, contents=True)
+    def __init__(self, obj_gi: "GalaxyInstance") -> None:
+        super().__init__(obj_gi=obj_gi)
+        gi_client = getattr(self.gi, self.CONTAINER_TYPE.API_MODULE)
+        get_fname = f"get_{self.CONTAINER_TYPE.API_MODULE}"
+        self._get_f = getattr(gi_client, get_fname)
+        show_fname = f"show_{self.CONTAINER_TYPE.__name__.lower()}"
+        self._show_f = getattr(gi_client, show_fname)
+
+    def get_previews(
+        self, name: Optional[str] = None, deleted: bool = False, **kwargs: Any
+    ) -> List[wrappers.DatasetContainerPreviewSubtype]:
+        dicts = self._get_f(name=name, deleted=deleted, **kwargs)
+        return [
+            cast(wrappers.DatasetContainerPreviewSubtype, self.CONTAINER_PREVIEW_TYPE(_, gi=self.obj_gi)) for _ in dicts
+        ]
+
+    def get(self, id_: str) -> wrappers.DatasetContainerSubtype:
+        """
+        Retrieve the dataset container corresponding to the given id.
+        """
+        cdict = self._show_f(id_)
+        c_infos = self._show_f(id_, contents=True)
         if not isinstance(c_infos, Sequence):
-            self._error(f'{show_fname}: unexpected reply: {c_infos!r}')
-        c_infos = [ctype.CONTENT_INFO_TYPE(_) for _ in c_infos]
-        return ctype(cdict, content_infos=c_infos, gi=self.obj_gi)
+            raise RuntimeError(f"{self._show_f.__name__}: unexpected reply: {c_infos!r}")
+        c_infos = [self.CONTAINER_TYPE.CONTENT_INFO_TYPE(_) for _ in c_infos]
+        return cast(wrappers.DatasetContainerSubtype, self.CONTAINER_TYPE(cdict, content_infos=c_infos, gi=self.obj_gi))
 
 
-class ObjLibraryClient(ObjDatasetContainerClient):
+class ObjLibraryClient(ObjDatasetContainerClient[wrappers.Library, wrappers.LibraryPreview]):
     """
     Interacts with Galaxy libraries.
     """
 
-    def __init__(self, obj_gi):
-        super().__init__(obj_gi)
+    CONTAINER_TYPE = wrappers.Library
+    CONTAINER_PREVIEW_TYPE = wrappers.LibraryPreview
 
-    def create(self, name, description=None, synopsis=None):
+    def create(self, name: str, description: Optional[str] = None, synopsis: Optional[str] = None) -> wrappers.Library:
         """
         Create a data library with the properties defined in the arguments.
 
@@ -118,23 +147,10 @@ class ObjLibraryClient(ObjDatasetContainerClient):
         :return: the library just created
         """
         res = self.gi.libraries.create_library(name, description, synopsis)
-        lib_info = self._get_dict('create_library', res)
-        return self.get(lib_info['id'])
+        lib_info = self._get_dict("create_library", res)
+        return self.get(lib_info["id"])
 
-    def get(self, id_):
-        """
-        Retrieve the data library corresponding to the given id.
-
-        :rtype: :class:`~.wrappers.Library`
-        :return: the library corresponding to ``id_``
-        """
-        return self._get_container(id_, wrappers.Library)
-
-    def get_previews(self, name=None, deleted=False):
-        dicts = self.gi.libraries.get_libraries(name=name, deleted=deleted)
-        return [wrappers.LibraryPreview(_, gi=self.obj_gi) for _ in dicts]
-
-    def list(self, name=None, deleted=False):
+    def list(self, name: Optional[str] = None, deleted: bool = False) -> List[wrappers.Library]:
         """
         Get libraries owned by the user of this Galaxy instance.
 
@@ -150,35 +166,35 @@ class ObjLibraryClient(ObjDatasetContainerClient):
             # return Library objects only for not-deleted libraries since Galaxy
             # does not filter them out and Galaxy release_14.08 and earlier
             # crashes when trying to get a deleted library
-            return [self.get(_['id']) for _ in dicts if not _['deleted']]
+            return [self.get(_["id"]) for _ in dicts if not _["deleted"]]
         else:
-            return [self.get(_['id']) for _ in dicts]
+            return [self.get(_["id"]) for _ in dicts]
 
-    def delete(self, id_=None, name=None):
+    def delete(self, id_: Optional[str] = None, name: Optional[str] = None) -> None:
         """
         Delete the library with the given id or name.
 
-        Note that the same name can map to multiple libraries.
+        Fails if multiple libraries have the specified name.
 
         .. warning::
           Deleting a data library is irreversible - all of the data from
           the library will be permanently deleted.
         """
-        for id_ in self._select_ids(id_=id_, name=name):
-            res = self.gi.libraries.delete_library(id_)
-            if not isinstance(res, Mapping):
-                self._error(f'delete_library: unexpected reply: {res!r}')
+        id_ = self._select_id(id_=id_, name=name)
+        res = self.gi.libraries.delete_library(id_)
+        if not isinstance(res, dict):
+            raise RuntimeError(f"delete_library: unexpected reply: {res!r}")
 
 
-class ObjHistoryClient(ObjDatasetContainerClient):
+class ObjHistoryClient(ObjDatasetContainerClient[wrappers.History, wrappers.HistoryPreview]):
     """
     Interacts with Galaxy histories.
     """
 
-    def __init__(self, obj_gi):
-        super().__init__(obj_gi)
+    CONTAINER_TYPE = wrappers.History
+    CONTAINER_PREVIEW_TYPE = wrappers.HistoryPreview
 
-    def create(self, name=None):
+    def create(self, name: Optional[str] = None) -> wrappers.History:
         """
         Create a new Galaxy history, optionally setting its name.
 
@@ -186,23 +202,10 @@ class ObjHistoryClient(ObjDatasetContainerClient):
         :return: the history just created
         """
         res = self.gi.histories.create_history(name=name)
-        hist_info = self._get_dict('create_history', res)
-        return self.get(hist_info['id'])
+        hist_info = self._get_dict("create_history", res)
+        return self.get(hist_info["id"])
 
-    def get(self, id_):
-        """
-        Retrieve the history corresponding to the given id.
-
-        :rtype: :class:`~.wrappers.History`
-        :return: the history corresponding to ``id_``
-        """
-        return self._get_container(id_, wrappers.History)
-
-    def get_previews(self, name=None, deleted=False):
-        dicts = self.gi.histories.get_histories(name=name, deleted=deleted)
-        return [wrappers.HistoryPreview(_, gi=self.obj_gi) for _ in dicts]
-
-    def list(self, name=None, deleted=False):
+    def list(self, name: Optional[str] = None, deleted: bool = False) -> List[wrappers.History]:
         """
         Get histories owned by the user of this Galaxy instance.
 
@@ -214,26 +217,26 @@ class ObjHistoryClient(ObjDatasetContainerClient):
         :rtype: list of :class:`~.wrappers.History`
         """
         dicts = self.gi.histories.get_histories(name=name, deleted=deleted)
-        return [self.get(_['id']) for _ in dicts]
+        return [self.get(_["id"]) for _ in dicts]
 
-    def delete(self, id_=None, name=None, purge=False):
+    def delete(self, id_: Optional[str] = None, name: Optional[str] = None, purge: bool = False) -> None:
         """
         Delete the history with the given id or name.
 
-        Note that the same name can map to multiple histories.
+        Fails if multiple histories have the same name.
 
         :type purge: bool
         :param purge: if ``True``, also purge (permanently delete) the history
 
         .. note::
-          For the purge option to work, the Galaxy instance must have the
+          The ``purge`` option works only if the Galaxy instance has the
           ``allow_user_dataset_purge`` option set to ``true`` in the
           ``config/galaxy.yml`` configuration file.
         """
-        for id_ in self._select_ids(id_=id_, name=name):
-            res = self.gi.histories.delete_history(id_, purge=purge)
-            if not isinstance(res, Mapping):
-                self._error(f'delete_history: unexpected reply: {res!r}')
+        id_ = self._select_id(id_=id_, name=name)
+        res = self.gi.histories.delete_history(id_, purge=purge)
+        if not isinstance(res, dict):
+            raise RuntimeError(f"delete_history: unexpected reply: {res!r}")
 
 
 class ObjWorkflowClient(ObjClient):
@@ -241,7 +244,7 @@ class ObjWorkflowClient(ObjClient):
     Interacts with Galaxy workflows.
     """
 
-    def import_new(self, src, publish=False):
+    def import_new(self, src: Union[str, Dict[str, Any]], publish: bool = False) -> wrappers.Workflow:
         """
         Imports a new workflow into Galaxy.
 
@@ -257,17 +260,17 @@ class ObjWorkflowClient(ObjClient):
         :rtype: :class:`~.wrappers.Workflow`
         :return: the workflow just imported
         """
-        if isinstance(src, Mapping):
+        if isinstance(src, dict):
             wf_dict = src
         else:
             try:
                 wf_dict = json.loads(src)
             except (TypeError, ValueError):
-                self._error(f'src not supported: {src!r}')
+                raise ValueError(f"src not supported: {src!r}")
         wf_info = self.gi.workflows.import_workflow_dict(wf_dict, publish)
-        return self.get(wf_info['id'])
+        return self.get(wf_info["id"])
 
-    def import_shared(self, id_):
+    def import_shared(self, id_: str) -> wrappers.Workflow:
         """
         Imports a shared workflow to the user's space.
 
@@ -278,9 +281,9 @@ class ObjWorkflowClient(ObjClient):
         :return: the workflow just imported
         """
         wf_info = self.gi.workflows.import_shared_workflow(id_)
-        return self.get(wf_info['id'])
+        return self.get(wf_info["id"])
 
-    def get(self, id_):
+    def get(self, id_: str) -> wrappers.Workflow:
         """
         Retrieve the workflow corresponding to the given id.
 
@@ -288,16 +291,18 @@ class ObjWorkflowClient(ObjClient):
         :return: the workflow corresponding to ``id_``
         """
         res = self.gi.workflows.show_workflow(id_)
-        wf_dict = self._get_dict('show_workflow', res)
+        wf_dict = self._get_dict("show_workflow", res)
         return wrappers.Workflow(wf_dict, gi=self.obj_gi)
 
     # the 'deleted' option is not available for workflows
-    def get_previews(self, name=None, published=False):
-        dicts = self.gi.workflows.get_workflows(name=name, published=published)
+    def get_previews(
+        self, name: Optional[str] = None, published: bool = False, **kwargs: Any
+    ) -> List[wrappers.WorkflowPreview]:
+        dicts = self.gi.workflows.get_workflows(name=name, published=published, **kwargs)
         return [wrappers.WorkflowPreview(_, gi=self.obj_gi) for _ in dicts]
 
     # the 'deleted' option is not available for workflows
-    def list(self, name=None, published=False):
+    def list(self, name: Optional[str] = None, published: bool = False) -> List[wrappers.Workflow]:
         """
         Get workflows owned by the user of this Galaxy instance.
 
@@ -309,29 +314,30 @@ class ObjWorkflowClient(ObjClient):
         :rtype: list of :class:`~.wrappers.Workflow`
         """
         dicts = self.gi.workflows.get_workflows(name=name, published=published)
-        return [self.get(_['id']) for _ in dicts]
+        return [self.get(_["id"]) for _ in dicts]
 
-    def delete(self, id_=None, name=None):
+    def delete(self, id_: Optional[str] = None, name: Optional[str] = None) -> None:
         """
         Delete the workflow with the given id or name.
 
-        Note that the same name can map to multiple workflows.
+        Fails if multiple workflows have the specified name.
 
         .. warning::
           Deleting a workflow is irreversible - all of the data from
           the workflow will be permanently deleted.
         """
-        for id_ in self._select_ids(id_=id_, name=name):
-            res = self.gi.workflows.delete_workflow(id_)
-            if not isinstance(res, str):
-                self._error(f"delete_workflow: unexpected reply: {res!r}")
+        id_ = self._select_id(id_=id_, name=name)
+        res = self.gi.workflows.delete_workflow(id_)
+        if not isinstance(res, str):
+            raise RuntimeError(f"delete_workflow: unexpected reply: {res!r}")
 
 
 class ObjInvocationClient(ObjClient):
     """
     Interacts with Galaxy Invocations.
     """
-    def get(self, id_) -> wrappers.Invocation:
+
+    def get(self, id_: str) -> wrappers.Invocation:
         """
         Get an invocation by ID.
 
@@ -341,22 +347,22 @@ class ObjInvocationClient(ObjClient):
         inv_dict = self.gi.invocations.show_invocation(id_)
         return wrappers.Invocation(inv_dict, self.obj_gi)
 
-    def get_previews(self) -> List[wrappers.InvocationPreview]:
+    def get_previews(self, **kwargs: Any) -> List[wrappers.InvocationPreview]:
         """
         Get previews of all invocations.
 
         :rtype: list of InvocationPreview
         :param: previews of invocations
         """
-        inv_list = self.gi.invocations.get_invocations()
-        return [wrappers.InvocationPreview(inv_dict, self.obj_gi) for inv_dict in inv_list]
+        inv_list = self.gi.invocations.get_invocations(**kwargs)
+        return [wrappers.InvocationPreview(inv_dict, gi=self.obj_gi) for inv_dict in inv_list]
 
     def list(
         self,
-        workflow=None,
-        history=None,
-        include_terminal=True,
-        limit=None
+        workflow: Optional[wrappers.Workflow] = None,
+        history: Optional[wrappers.History] = None,
+        include_terminal: bool = True,
+        limit: Optional[int] = None,
     ) -> List[wrappers.Invocation]:
         """
         Get full listing of workflow invocations, or select a subset
@@ -366,7 +372,7 @@ class ObjInvocationClient(ObjClient):
         :param workflow: Include only invocations associated with
           this workflow
 
-        :type history: str
+        :type history: wrappers.History
         :param history: Include only invocations associated with
           this history
 
@@ -385,8 +391,8 @@ class ObjInvocationClient(ObjClient):
             history_id=history.id if history else None,
             include_terminal=include_terminal,
             limit=limit,
-            view='element',
-            step_details=True
+            view="element",
+            step_details=True,
         )
         return [wrappers.Invocation(inv_dict, self.obj_gi) for inv_dict in inv_dict_list]
 
@@ -396,7 +402,7 @@ class ObjToolClient(ObjClient):
     Interacts with Galaxy tools.
     """
 
-    def get(self, id_, io_details=False, link_details=False):
+    def get(self, id_: str, io_details: bool = False, link_details: bool = False) -> wrappers.Tool:
         """
         Retrieve the tool corresponding to the given id.
 
@@ -409,12 +415,11 @@ class ObjToolClient(ObjClient):
         :rtype: :class:`~.wrappers.Tool`
         :return: the tool corresponding to ``id_``
         """
-        res = self.gi.tools.show_tool(id_, io_details=io_details,
-                                      link_details=link_details)
-        tool_dict = self._get_dict('show_tool', res)
+        res = self.gi.tools.show_tool(id_, io_details=io_details, link_details=link_details)
+        tool_dict = self._get_dict("show_tool", res)
         return wrappers.Tool(tool_dict, gi=self.obj_gi)
 
-    def get_previews(self, name=None, trackster=None):
+    def get_previews(self, name: Optional[str] = None, trackster: bool = False, **kwargs: Any) -> List[wrappers.Tool]:
         """
         Get the list of tools installed on the Galaxy instance.
 
@@ -427,11 +432,11 @@ class ObjToolClient(ObjClient):
 
         :rtype: list of :class:`~.wrappers.Tool`
         """
-        dicts = self.gi.tools.get_tools(name=name, trackster=trackster)
+        dicts = self.gi.tools.get_tools(name=name, trackster=trackster, **kwargs)
         return [wrappers.Tool(_, gi=self.obj_gi) for _ in dicts]
 
     # the 'deleted' option is not available for tools
-    def list(self, name=None, trackster=None):
+    def list(self, name: Optional[str] = None, trackster: bool = False) -> List[wrappers.Tool]:
         """
         Get the list of tools installed on the Galaxy instance.
 
@@ -461,7 +466,7 @@ class ObjJobClient(ObjClient):
     Interacts with Galaxy jobs.
     """
 
-    def get(self, id_, full_details=False):
+    def get(self, id_: str, full_details: bool = False) -> wrappers.Job:
         """
         Retrieve the job corresponding to the given id.
 
@@ -473,18 +478,84 @@ class ObjJobClient(ObjClient):
         :return: the job corresponding to ``id_``
         """
         res = self.gi.jobs.show_job(id_, full_details)
-        job_dict = self._get_dict('job_tool', res)
+        job_dict = self._get_dict("show_job", res)
         return wrappers.Job(job_dict, gi=self.obj_gi)
 
-    def get_previews(self):
-        dicts = self.gi.jobs.get_jobs()
+    def get_previews(self, **kwargs: Any) -> List[wrappers.JobPreview]:
+        dicts = self.gi.jobs.get_jobs(**kwargs)
         return [wrappers.JobPreview(_, gi=self.obj_gi) for _ in dicts]
 
-    def list(self):
+    def list(self) -> List[wrappers.Job]:
         """
         Get the list of jobs of the current user.
 
         :rtype: list of :class:`~.wrappers.Job`
         """
         dicts = self.gi.jobs.get_jobs()
-        return [self.get(_['id']) for _ in dicts]
+        return [self.get(_["id"]) for _ in dicts]
+
+
+class ObjDatasetClient(ObjClient):
+    """
+    Interacts with Galaxy datasets.
+    """
+
+    @overload
+    def get(self, id_: str, hda_ldda: Literal["hda"] = "hda") -> wrappers.HistoryDatasetAssociation:
+        ...
+
+    @overload
+    def get(self, id_: str, hda_ldda: Literal["ldda"]) -> wrappers.LibraryDatasetDatasetAssociation:
+        ...
+
+    def get(self, id_: str, hda_ldda: HdaLdda = "hda") -> wrappers.Dataset:
+        """
+        Retrieve the dataset corresponding to the given id.
+
+        :type hda_ldda: str
+        :param hda_ldda: Whether to show a history dataset ('hda' - the default)
+          or library dataset ('ldda')
+
+        :rtype: :class:`~.wrappers.HistoryDatasetAssociation` or :class:`~.wrappers.LibraryDatasetDatasetAssociation`
+        :return: the history or library dataset corresponding to ``id_``
+        """
+        res = self.gi.datasets.show_dataset(id_, hda_ldda=hda_ldda)
+        ds_dict = self._get_dict("show_dataset", res)
+        if hda_ldda == "hda":
+            hist = self.obj_gi.histories.get(ds_dict["history_id"])
+            return wrappers.HistoryDatasetAssociation(ds_dict, hist, gi=self.obj_gi)
+        elif hda_ldda == "ldda":
+            lib = self.obj_gi.libraries.get(ds_dict["parent_library_id"])
+            return wrappers.LibraryDatasetDatasetAssociation(ds_dict, lib, gi=self.obj_gi)
+        else:
+            raise ValueError(f"Unsupported value for hda_ldda: {hda_ldda}")
+
+    def get_previews(self, **kwargs: Any) -> list:
+        raise NotImplementedError()
+
+    def list(self) -> list:
+        raise NotImplementedError()
+
+
+class ObjDatasetCollectionClient(ObjClient):
+    """
+    Interacts with Galaxy dataset collections.
+    """
+
+    def get(self, id_: str) -> wrappers.HistoryDatasetCollectionAssociation:
+        """
+        Retrieve the dataset collection corresponding to the given id.
+
+        :rtype: :class:`~.wrappers.HistoryDatasetCollectionAssociation`
+        :return: the history dataset collection corresponding to ``id_``
+        """
+        res = self.gi.dataset_collections.show_dataset_collection(id_)
+        ds_dict = self._get_dict("show_dataset_collection", res)
+        hist = self.obj_gi.histories.get(ds_dict["history_id"])
+        return wrappers.HistoryDatasetCollectionAssociation(ds_dict, hist, gi=self.obj_gi)
+
+    def get_previews(self, **kwargs: Any) -> list:
+        raise NotImplementedError()
+
+    def list(self) -> list:
+        raise NotImplementedError()
