@@ -1,16 +1,32 @@
 """
 Contains possible interactions with the Galaxy Quota
 """
+
+from typing import (
+    Any,
+    Dict,
+    List,
+    Optional,
+    TYPE_CHECKING,
+)
+
+from typing_extensions import Literal
+
 from bioblend.galaxy.client import Client
 
+if TYPE_CHECKING:
+    from bioblend.galaxy import GalaxyInstance
+
+QuotaOperations = Literal["+", "-", "="]
+
 
 class QuotaClient(Client):
-    module = 'quotas'
+    module = "quotas"
 
-    def __init__(self, galaxy_instance):
+    def __init__(self, galaxy_instance: "GalaxyInstance") -> None:
         super().__init__(galaxy_instance)
 
-    def get_quotas(self, deleted=False):
+    def get_quotas(self, deleted: bool = False) -> List[Dict[str, Any]]:
         """
         Get a list of quotas
 
@@ -32,7 +48,7 @@ class QuotaClient(Client):
         """
         return self._get(deleted=deleted)
 
-    def show_quota(self, quota_id, deleted=False):
+    def show_quota(self, quota_id: str, deleted: bool = False) -> Dict[str, Any]:
         """
         Display information on a quota
 
@@ -59,8 +75,16 @@ class QuotaClient(Client):
         """
         return self._get(id=quota_id, deleted=deleted)
 
-    def create_quota(self, name, description, amount, operation,
-                     default='no', in_users=None, in_groups=None):
+    def create_quota(
+        self,
+        name: str,
+        description: str,
+        amount: str,
+        operation: QuotaOperations,
+        default: Optional[Literal["no", "registered", "unregistered"]] = "no",
+        in_users: Optional[List[str]] = None,
+        in_groups: Optional[List[str]] = None,
+    ) -> Dict[str, Any]:
         """
         Create a new quota
 
@@ -97,23 +121,33 @@ class QuotaClient(Client):
              'id': '386f14984287a0f7',
              'name': 'Testing'}
         """
-        payload = {
-            'name': name,
-            'description': description,
-            'amount': amount,
-            'operation': operation,
-            'default': default
+        payload: Dict[str, Any] = {
+            "name": name,
+            "description": description,
+            "amount": amount,
+            "operation": operation,
+            "default": default,
         }
         if in_users:
-            payload['in_users'] = in_users
+            payload["in_users"] = in_users
 
         if in_groups:
-            payload['in_groups'] = in_groups
+            payload["in_groups"] = in_groups
 
         return self._post(payload)
 
-    def update_quota(self, quota_id, name=None, description=None, amount=None, operation=None,
-                     default='no', in_users=None, in_groups=None):
+    def update_quota(
+        self,
+        quota_id: str,
+        name: Optional[str] = None,
+        description: Optional[str] = None,
+        amount: Optional[str] = None,
+        operation: Optional[QuotaOperations] = None,
+        default: str = "no",
+        in_users: Optional[List[str]] = None,
+        in_groups: Optional[List[str]] = None,
+    ) -> str:
+
         """
         Update an existing quota
 
@@ -154,30 +188,28 @@ class QuotaClient(Client):
 
             "Quota 'Testing-A' has been renamed to 'Testing-B'; Quota 'Testing-e' is now '-100.0 GB'; Quota 'Testing-B' is now the default for unregistered users"
         """
-        payload = {
-            'default': default
-        }
+        payload: Dict[str, Any] = {"default": default}
         if name:
-            payload['name'] = name
+            payload["name"] = name
 
         if description:
-            payload['description'] = description
+            payload["description"] = description
 
         if amount:
-            payload['amount'] = amount
+            payload["amount"] = amount
 
         if operation:
-            payload['operation'] = operation
+            payload["operation"] = operation
 
         if in_users:
-            payload['in_users'] = in_users
+            payload["in_users"] = in_users
 
         if in_groups:
-            payload['in_groups'] = in_groups
+            payload["in_groups"] = in_groups
 
         return self._put(id=quota_id, payload=payload)
 
-    def delete_quota(self, quota_id):
+    def delete_quota(self, quota_id: str) -> str:
         """
         Delete a quota
 
@@ -194,7 +226,7 @@ class QuotaClient(Client):
         """
         return self._delete(id=quota_id)
 
-    def undelete_quota(self, quota_id):
+    def undelete_quota(self, quota_id: str) -> str:
         """
         Undelete a quota
 
@@ -207,5 +239,5 @@ class QuotaClient(Client):
 
             "Undeleted 1 quotas: Testing-B"
         """
-        url = self._make_url(quota_id, deleted=True) + '/undelete'
-        return self._post(url=url, payload={'id': quota_id})
+        url = self._make_url(quota_id, deleted=True) + "/undelete"
+        return self._post(url=url, payload={"id": quota_id})
