@@ -1,16 +1,35 @@
 """
 Contains interactions dealing with Galaxy dependency resolvers.
 """
+from typing import (
+    List,
+    Optional,
+    TYPE_CHECKING,
+)
+
+from typing_extensions import Literal
+
 from bioblend.galaxy.client import Client
 
+if TYPE_CHECKING:
+    from bioblend.galaxy import GalaxyInstance
+
 
 class ToolDependenciesClient(Client):
-    module = 'dependency_resolvers'
+    module = "dependency_resolvers"
 
-    def __init__(self, galaxy_instance):
+    def __init__(self, galaxy_instance: "GalaxyInstance") -> None:
         super().__init__(galaxy_instance)
 
-    def summarize_toolbox(self, index=None, tool_ids=None, resolver_type=None, include_containers=False, container_type=None, index_by='requirements'):
+    def summarize_toolbox(
+        self,
+        index: Optional[int] = None,
+        tool_ids: Optional[List[str]] = None,
+        resolver_type: Optional[str] = None,
+        include_containers: bool = False,
+        container_type: Optional[str] = None,
+        index_by: Literal["requirements", "tools"] = "requirements",
+    ) -> list:
         """
         Summarize requirements across toolbox (for Tool Management grid).
 
@@ -66,19 +85,19 @@ class ToolDependenciesClient(Client):
           Galaxy admin. It relies on an experimental API particularly tied to
           the GUI and therefore is subject to breaking changes.
         """
-        assert index_by in ['tools', 'requirements'], "index_by must be one of 'tools' or 'requirements'."
+        assert index_by in ["tools", "requirements"], "index_by must be one of 'tools' or 'requirements'."
         params = {
-            'include_containers': str(include_containers),
-            'index_by': index_by,
+            "include_containers": str(include_containers),
+            "index_by": index_by,
         }
         if index:
-            params['index'] = str(index)
+            params["index"] = str(index)
         if tool_ids:
-            params['tool_ids'] = ','.join(tool_ids)
+            params["tool_ids"] = ",".join(tool_ids)
         if resolver_type:
-            params['resolver_type'] = resolver_type
+            params["resolver_type"] = resolver_type
         if container_type:
-            params['container_type'] = container_type
+            params["container_type"] = container_type
 
-        url = '/'.join((self._make_url(), 'toolbox'))
+        url = "/".join((self._make_url(), "toolbox"))
         return self._get(url=url, params=params)
