@@ -3,19 +3,36 @@ Contains possible interaction dealing with Galaxy tools.
 """
 import warnings
 from os.path import basename
-from typing import List
+from typing import (
+    Any,
+    Dict,
+    List,
+    Optional,
+    TYPE_CHECKING,
+    Union,
+)
+
+from typing_extensions import Literal
 
 from bioblend.galaxy.client import Client
+from bioblend.galaxyclient import UPLOAD_CHUNK_SIZE
 from bioblend.util import attach_file
+from .inputs import InputsBuilder
+
+if TYPE_CHECKING:
+    from bioblend.galaxy import GalaxyInstance
 
 
 class ToolClient(Client):
-    module = 'tools'
+    gi: "GalaxyInstance"
+    module = "tools"
 
-    def __init__(self, galaxy_instance):
+    def __init__(self, galaxy_instance: "GalaxyInstance") -> None:
         super().__init__(galaxy_instance)
 
-    def get_tools(self, tool_id=None, name=None, trackster=None):
+    def get_tools(
+        self, tool_id: Optional[str] = None, name: Optional[str] = None, trackster: Optional[bool] = None
+    ) -> List[Dict[str, Any]]:
         """
         Get all tools, or select a subset by specifying optional arguments for
         filtering (e.g. a tool name).
@@ -41,20 +58,20 @@ class ToolClient(Client):
         """
         if tool_id is not None:
             warnings.warn(
-                'The tool_id parameter is deprecated, use the show_tool() method to view details of a tool for which you know the ID.',
-                category=FutureWarning
+                "The tool_id parameter is deprecated, use the show_tool() method to view details of a tool for which you know the ID.",
+                category=FutureWarning,
             )
         if tool_id is not None and name is not None:
-            raise ValueError('Provide only one argument between name or tool_id, but not both')
+            raise ValueError("Provide only one argument between name or tool_id, but not both")
         tools = self._raw_get_tool(in_panel=False, trackster=trackster)
         if tool_id is not None:
-            tool = next((_ for _ in tools if _['id'] == tool_id), None)
+            tool = next((_ for _ in tools if _["id"] == tool_id), None)
             tools = [tool] if tool is not None else []
         elif name is not None:
-            tools = [_ for _ in tools if _['name'] == name]
+            tools = [_ for _ in tools if _["name"] == name]
         return tools
 
-    def get_tool_panel(self):
+    def get_tool_panel(self) -> List[Dict[str, Any]]:
         """
         Get a list of available tool elements in Galaxy's configured toolbox.
 
@@ -66,13 +83,14 @@ class ToolClient(Client):
         """
         return self._raw_get_tool(in_panel=True)
 
-    def _raw_get_tool(self, in_panel=None, trackster=None):
-        params = {}
-        params['in_panel'] = in_panel
-        params['trackster'] = trackster
+    def _raw_get_tool(self, in_panel: Optional[bool] = None, trackster: Optional[bool] = None) -> List[Dict[str, Any]]:
+        params = {
+            "in_panel": in_panel,
+            "trackster": trackster,
+        }
         return self._get(params=params)
 
-    def requirements(self, tool_id):
+    def requirements(self, tool_id: str) -> List[Dict[str, Any]]:
         """
         Return the resolver status for a specific tool.
 
@@ -104,7 +122,7 @@ class ToolClient(Client):
         .. note::
           This method works only if the user is a Galaxy admin.
         """
-        url = self._make_url(tool_id) + '/requirements'
+        url = self._make_url(tool_id) + "/requirements"
         return self._get(url=url)
 
     def reload(self, tool_id: str) -> dict:
@@ -128,7 +146,7 @@ class ToolClient(Client):
         .. note::
           This method works only if the user is a Galaxy admin.
         """
-        url = self._make_url(tool_id) + '/reload'
+        url = self._make_url(tool_id) + "/reload"
         return self._put(url=url)
 
     def get_citations(self, tool_id: str) -> List[dict]:
@@ -141,10 +159,10 @@ class ToolClient(Client):
         :rtype: list of dicts
         :param: list containing the citations
         """
-        url = self._make_url(tool_id) + '/citations'
+        url = self._make_url(tool_id) + "/citations"
         return self._get(url=url)
 
-    def install_dependencies(self, tool_id):
+    def install_dependencies(self, tool_id: str) -> List[Dict[str, Any]]:
         """
         Install dependencies for a given tool via a resolver.
         This works only for Conda currently.
@@ -152,13 +170,13 @@ class ToolClient(Client):
         :type tool_id: str
         :param tool_id: id of the requested tool
 
-        :rtype: dict
-        :return: Tool requirement status
+        :rtype: list of dicts
+        :return: List of tool requirement status dictionaries
 
         .. note::
           This method works only if the user is a Galaxy admin.
         """
-        url = self._make_url(tool_id) + '/install_dependencies'
+        url = self._make_url(tool_id) + "/install_dependencies"
         return self._post(url=url)
 
     def uninstall_dependencies(self, tool_id: str) -> dict:
@@ -175,10 +193,10 @@ class ToolClient(Client):
         .. note::
           This method works only if the user is a Galaxy admin.
         """
-        url = self._make_url(tool_id) + '/dependencies'
+        url = self._make_url(tool_id) + "/dependencies"
         return self._delete(url=url)
 
-    def show_tool(self, tool_id, io_details=False, link_details=False):
+    def show_tool(self, tool_id: str, io_details: bool = False, link_details: bool = False) -> Dict[str, Any]:
         """
         Get details of a given tool.
 
@@ -194,12 +212,19 @@ class ToolClient(Client):
         :rtype: dict
         :return: Information about the tool's interface
         """
-        params = {}
-        params['io_details'] = io_details
-        params['link_details'] = link_details
+        params = {
+            "io_details": io_details,
+            "link_details": link_details,
+        }
         return self._get(id=tool_id, params=params)
 
-    def build(self, tool_id, inputs=None, tool_version=None, history_id=None):
+    def build(
+        self,
+        tool_id: str,
+        inputs: Optional[Dict[str, Any]] = None,
+        tool_version: Optional[str] = None,
+        history_id: Optional[str] = None,
+    ) -> Dict[str, Any]:
         """
         This method returns the tool model, which includes an updated input parameter array for the given tool,
         based on user-defined "inputs".
@@ -297,22 +322,29 @@ class ToolClient(Client):
             }
 
         """
-        params = {}
+        params: Dict[str, Union[str, Dict]] = {}
 
         if inputs:
-            params['inputs'] = inputs
+            params["inputs"] = inputs
 
         if tool_version:
-            params['tool_version'] = tool_version
+            params["tool_version"] = tool_version
 
         if history_id:
-            params['history_id'] = history_id
+            params["history_id"] = history_id
 
-        url = '/'.join((self.gi.url, 'tools', tool_id, 'build'))
+        url = "/".join((self.gi.url, "tools", tool_id, "build"))
 
         return self._post(payload=params, url=url)
 
-    def run_tool(self, history_id, tool_id, tool_inputs, input_format='legacy'):
+    def run_tool(
+        self,
+        history_id: str,
+        tool_id: str,
+        tool_inputs: Union[InputsBuilder, dict],
+        input_format: Literal["21.01", "legacy"] = "legacy",
+    ) -> Dict[str, Any]:
+
         """
         Runs tool specified by ``tool_id`` in history indicated
         by ``history_id`` with inputs from ``dict`` ``tool_inputs``.
@@ -378,21 +410,34 @@ class ToolClient(Client):
                           'visible': True}]}
 
         The ``tool_inputs`` dict should contain input datasets and parameters
-        in the (largely undocumented) format used by the Galaxy API.
-        Some examples can be found in `Galaxy's API test suite
+        in the (largely undocumented) format used by the Galaxy API. If you are unsure
+        how to construct this dict for the tool you want to run, you can obtain a
+        template by executing the ``build()`` method and taking the value of
+        ``state_inputs`` from its output, then modifying it as you require.
+        You can also check the examples in `Galaxy's API test suite
         <https://github.com/galaxyproject/galaxy/blob/dev/lib/galaxy_test/api/test_tools.py>`_.
         """
-        payload = {}
-        payload["history_id"] = history_id
-        payload["tool_id"] = tool_id
-        payload["input_format"] = input_format
-        try:
+        payload: Dict[str, Union[str, Dict]] = {
+            "history_id": history_id,
+            "tool_id": tool_id,
+            "input_format": input_format,
+        }
+
+        if isinstance(tool_inputs, InputsBuilder):
             payload["inputs"] = tool_inputs.to_dict()
-        except AttributeError:
+        else:
             payload["inputs"] = tool_inputs
         return self._post(payload)
 
-    def upload_file(self, path, history_id, **keywords):
+    def upload_file(
+        self,
+        path: str,
+        history_id: str,
+        storage: Optional[str] = None,
+        metadata: Optional[dict] = None,
+        chunk_size: Optional[int] = UPLOAD_CHUNK_SIZE,
+        **kwargs: Any,
+    ) -> Dict[str, Any]:
         """
         Upload the file specified by ``path`` to the history specified by
         ``history_id``.
@@ -403,6 +448,15 @@ class ToolClient(Client):
         :type history_id: str
         :param history_id: id of the history where to upload the file
 
+        :type storage: str
+        :param storage: Local path to store URLs resuming uploads
+
+        :type metadata: dict
+        :param metadata: Metadata to send with upload request
+
+        :type chunk_size: int
+        :param chunk_size: Number of bytes to send in each chunk
+
         :type file_name: str
         :param file_name: (optional) name of the new history dataset
 
@@ -421,19 +475,60 @@ class ToolClient(Client):
         :param space_to_tab: whether to convert spaces to tabs. Default is
           ``False``. Applicable only if to_posix_lines is ``True``
 
+        :type auto_decompress: bool
+        :param auto_decompress: Automatically decompress files if the uploaded
+          file is compressed and the file type is not one that supports
+          compression (e.g. ``fastqsanger.gz``). Default is ``False``.
+
+        :rtype: dict
+        :return: Information about the created upload job
+
+        .. note::
+          The following parameters work only on Galaxy 22.01 or later:
+          ``storage``, ``metadata``, ``chunk_size``, ``auto_decompress``.
+        """
+        if self.gi.config.get_version()["version_major"] >= "22.01":
+            # Use the tus protocol
+            uploader = self.gi.get_tus_uploader(path, storage=storage, metadata=metadata, chunk_size=chunk_size)
+            uploader.upload()
+            return self.post_to_fetch(path, history_id, uploader.session_id, **kwargs)
+        else:
+            if "file_name" not in kwargs:
+                kwargs["file_name"] = basename(path)
+            payload = self._upload_payload(history_id, **kwargs)
+            payload["files_0|file_data"] = attach_file(path, name=kwargs["file_name"])
+            try:
+                return self._post(payload, files_attached=True)
+            finally:
+                payload["files_0|file_data"].close()
+
+    def post_to_fetch(self, path: str, history_id: str, session_id: str, **kwargs: Any) -> Dict[str, Any]:
+        """
+        Make a POST request to the Fetch API after performing a tus upload.
+
+        This is called by :meth:`upload_file` after performing an upload. This
+        method is useful if you want to control the tus uploader yourself (e.g.
+        to report on progress)::
+
+            uploader = gi.get_tus_uploader(path, storage=storage)
+            while uploader.offset < uploader.file_size:
+                uploader.upload_chunk()
+                # perform other actions...
+            gi.tools.post_to_fetch(path, history_id, uploader.session_id, **upload_kwargs)
+
+        :type session_id: str
+        :param session_id: Session ID returned by the tus service
+
+        See :meth:`upload_file` for additional parameters.
+
         :rtype: dict
         :return: Information about the created upload job
         """
-        if "file_name" not in keywords:
-            keywords["file_name"] = basename(path)
-        payload = self._upload_payload(history_id, **keywords)
-        payload["files_0|file_data"] = attach_file(path, name=keywords["file_name"])
-        try:
-            return self._post(payload, files_attached=True)
-        finally:
-            payload["files_0|file_data"].close()
+        payload = self._fetch_payload(path, history_id, session_id, **kwargs)
+        url = "/".join((self.gi.url, "tools/fetch"))
+        return self._post(payload, url=url)
 
-    def upload_from_ftp(self, path, history_id, **keywords):
+    def upload_from_ftp(self, path: str, history_id: str, **kwargs: Any) -> Dict[str, Any]:
         """
         Upload the file specified by ``path`` from the user's FTP directory to
         the history specified by ``history_id``.
@@ -449,11 +544,11 @@ class ToolClient(Client):
         :rtype: dict
         :return: Information about the created upload job
         """
-        payload = self._upload_payload(history_id, **keywords)
-        payload['files_0|ftp_files'] = path
+        payload = self._upload_payload(history_id, **kwargs)
+        payload["files_0|ftp_files"] = path
         return self._post(payload)
 
-    def paste_content(self, content, history_id, **kwds):
+    def paste_content(self, content: str, history_id: str, **kwargs: Any) -> Dict[str, Any]:
         """
         Upload a string to a new dataset in the history specified by
         ``history_id``.
@@ -470,25 +565,49 @@ class ToolClient(Client):
 
         See :meth:`upload_file` for the optional parameters.
         """
-        payload = self._upload_payload(history_id, **kwds)
+        payload = self._upload_payload(history_id, **kwargs)
         payload["files_0|url_paste"] = content
         return self._post(payload, files_attached=False)
 
     put_url = paste_content
 
-    def _upload_payload(self, history_id, **keywords):
-        payload = {}
-        payload["history_id"] = history_id
-        payload["tool_id"] = keywords.get("tool_id", "upload1")
-        tool_input = {}
-        tool_input["file_type"] = keywords.get('file_type', 'auto')
-        tool_input["dbkey"] = keywords.get("dbkey", "?")
-        if not keywords.get('to_posix_lines', True):
-            tool_input['files_0|to_posix_lines'] = False
-        elif keywords.get('space_to_tab', False):
-            tool_input['files_0|space_to_tab'] = 'Yes'
-        if 'file_name' in keywords:
-            tool_input["files_0|NAME"] = keywords['file_name']
-        tool_input["files_0|type"] = "upload_dataset"
-        payload["inputs"] = tool_input
+    def _upload_payload(self, history_id: str, **kwargs: Any) -> Dict[str, Any]:
+        tool_input: Dict[str, Any] = {
+            "file_type": kwargs.get("file_type", "auto"),
+            "dbkey": kwargs.get("dbkey", "?"),
+            "files_0|type": "upload_dataset",
+        }
+        if not kwargs.get("to_posix_lines", True):
+            tool_input["files_0|to_posix_lines"] = False
+        elif kwargs.get("space_to_tab", False):
+            tool_input["files_0|space_to_tab"] = "Yes"
+        if "file_name" in kwargs:
+            tool_input["files_0|NAME"] = kwargs["file_name"]
+        return {
+            "history_id": history_id,
+            "tool_id": kwargs.get("tool_id", "upload1"),
+            "inputs": tool_input,
+        }
+
+    def _fetch_payload(self, path: str, history_id: str, session_id: str, **kwargs: Any) -> dict:
+        file_name = kwargs.get("file_name", basename(path))
+        element = {
+            "src": "files",
+            "ext": kwargs.get("file_type", "auto"),
+            "dbkey": kwargs.get("dbkey", "?"),
+            "to_posix_lines": kwargs.get("to_posix_lines", True),
+            "space_to_tab": kwargs.get("space_to_tab", False),
+            "name": file_name,
+        }
+        payload = {
+            "history_id": history_id,
+            "targets": [
+                {
+                    "destination": {"type": "hdas"},
+                    "elements": [element],
+                }
+            ],
+            "files_0|file_data": {"session_id": session_id, "name": file_name},
+            "auto_decompress": kwargs.get("auto_decompress", False),
+        }
         return payload
