@@ -7,20 +7,31 @@ import warnings
 from typing import (
     Any,
     Dict,
+    List,
     Optional,
+    TYPE_CHECKING,
 )
 
+from typing_extensions import Literal
+
 from bioblend.galaxy.client import Client
 
+if TYPE_CHECKING:
+    from bioblend.galaxy import GalaxyInstance
+
+InputsBy = Literal["step_index|step_uuid", "step_index", "step_id", "step_uuid", "name"]  # type: ignore[name-defined]
+
 
 class WorkflowClient(Client):
-    module = 'workflows'
+    module = "workflows"
 
-    def __init__(self, galaxy_instance):
+    def __init__(self, galaxy_instance: "GalaxyInstance") -> None:
         super().__init__(galaxy_instance)
 
     # the 'deleted' option is not available for workflows
-    def get_workflows(self, workflow_id=None, name=None, published=False):
+    def get_workflows(
+        self, workflow_id: Optional[str] = None, name: Optional[str] = None, published: bool = False
+    ) -> List[Dict[str, Any]]:
         """
         Get all workflows, or select a subset by specifying optional arguments
         for filtering (e.g. a workflow name).
@@ -49,23 +60,23 @@ class WorkflowClient(Client):
         """
         if workflow_id is not None:
             warnings.warn(
-                'The workflow_id parameter is deprecated, use the show_workflow() method to view details of a workflow for which you know the ID.',
-                category=FutureWarning
+                "The workflow_id parameter is deprecated, use the show_workflow() method to view details of a workflow for which you know the ID.",
+                category=FutureWarning,
             )
         if workflow_id is not None and name is not None:
-            raise ValueError('Provide only one argument between name or workflow_id, but not both')
-        params = {}
+            raise ValueError("Provide only one argument between name or workflow_id, but not both")
+        params: Dict[str, Any] = {}
         if published:
-            params['show_published'] = True
+            params["show_published"] = True
         workflows = self._get(params=params)
         if workflow_id is not None:
-            workflow = next((_ for _ in workflows if _['id'] == workflow_id), None)
+            workflow = next((_ for _ in workflows if _["id"] == workflow_id), None)
             workflows = [workflow] if workflow is not None else []
         elif name is not None:
-            workflows = [_ for _ in workflows if _['name'] == name]
+            workflows = [_ for _ in workflows if _["name"] == name]
         return workflows
 
-    def show_workflow(self, workflow_id, version=None):
+    def show_workflow(self, workflow_id: str, version: Optional[int] = None) -> Dict[str, Any]:
         """
         Display information needed to run a workflow.
 
@@ -84,13 +95,13 @@ class WorkflowClient(Client):
              'name': 'Simple',
              'url': '/api/workflows/92c56938c2f9b315'}
         """
-        params = {}
+        params: Dict[str, Any] = {}
         if version is not None:
-            params['version'] = version
+            params["version"] = version
 
         return self._get(id=workflow_id, params=params)
 
-    def get_workflow_inputs(self, workflow_id, label):
+    def get_workflow_inputs(self, workflow_id: str, label: str) -> List[str]:
         """
         Get a list of workflow input IDs that match the given label.
         If no input matches the given label, an empty list is returned.
@@ -105,10 +116,10 @@ class WorkflowClient(Client):
         :return: list of workflow inputs matching the label query
         """
         wf = self._get(id=workflow_id)
-        inputs = wf['inputs']
-        return [id for id in inputs if inputs[id]['label'] == label]
+        inputs = wf["inputs"]
+        return [id for id in inputs if inputs[id]["label"] == label]
 
-    def import_workflow_dict(self, workflow_dict, publish=False):
+    def import_workflow_dict(self, workflow_dict: Dict[str, Any], publish: bool = False) -> Dict[str, Any]:
         """
         Imports a new workflow given a dictionary representing a previously
         exported workflow.
@@ -135,12 +146,12 @@ class WorkflowClient(Client):
              'model_class': 'StoredWorkflow',
              'id': '94bac0a90086bdcf'}
         """
-        payload = {'workflow': workflow_dict, 'publish': publish}
+        payload = {"workflow": workflow_dict, "publish": publish}
 
         url = self._make_url() + "/upload"
         return self._post(url=url, payload=payload)
 
-    def import_workflow_from_local_path(self, file_local_path, publish=False):
+    def import_workflow_from_local_path(self, file_local_path: str, publish: bool = False) -> Dict[str, Any]:
         """
         Imports a new workflow given the path to a file containing a previously
         exported workflow.
@@ -173,7 +184,7 @@ class WorkflowClient(Client):
 
         return self.import_workflow_dict(workflow_json, publish)
 
-    def import_shared_workflow(self, workflow_id):
+    def import_shared_workflow(self, workflow_id: str) -> Dict[str, Any]:
         """
         Imports a new workflow from the shared published workflows.
 
@@ -191,11 +202,11 @@ class WorkflowClient(Client):
              'tags': [],
              'url': '/api/workflows/ee0e2b4b696d9092'}
         """
-        payload = {'shared_workflow_id': workflow_id}
+        payload = {"shared_workflow_id": workflow_id}
         url = self._make_url()
         return self._post(url=url, payload=payload)
 
-    def export_workflow_dict(self, workflow_id, version=None):
+    def export_workflow_dict(self, workflow_id: str, version: Optional[int] = None) -> Dict[str, Any]:
         """
         Exports a workflow.
 
@@ -208,14 +219,16 @@ class WorkflowClient(Client):
         :rtype: dict
         :return: Dictionary representing the requested workflow
         """
-        params = {}
+        params: Dict[str, Any] = {}
         if version is not None:
-            params['version'] = version
+            params["version"] = version
 
-        url = '/'.join((self._make_url(), 'download', workflow_id))
+        url = "/".join((self._make_url(), "download", workflow_id))
         return self._get(url=url, params=params)
 
-    def export_workflow_to_local_path(self, workflow_id, file_local_path, use_default_filename=True):
+    def export_workflow_to_local_path(
+        self, workflow_id: str, file_local_path: str, use_default_filename: bool = True
+    ) -> None:
         """
         Exports a workflow in JSON format to a given local path.
 
@@ -241,10 +254,10 @@ class WorkflowClient(Client):
             filename = f"Galaxy-Workflow-{workflow_dict['name']}.ga"
             file_local_path = os.path.join(file_local_path, filename)
 
-        with open(file_local_path, 'w') as fp:
+        with open(file_local_path, "w") as fp:
             json.dump(workflow_dict, fp)
 
-    def update_workflow(self, workflow_id, **kwds):
+    def update_workflow(self, workflow_id: str, **kwargs: Any) -> Dict[str, Any]:
         """
         Update a given workflow.
 
@@ -272,13 +285,22 @@ class WorkflowClient(Client):
         :rtype: dict
         :return: Dictionary representing the updated workflow
         """
-        return self._put(payload=kwds, id=workflow_id)
+        return self._put(payload=kwargs, id=workflow_id)
 
-    def invoke_workflow(self, workflow_id: str, inputs: Optional[dict] = None,
-                        params: Optional[dict] = None, history_id: Optional[str] = None,
-                        history_name: Optional[str] = None, import_inputs_to_history: bool = False,
-                        replacement_params: Optional[dict] = None, allow_tool_state_corrections: bool = False,
-                        inputs_by: Optional[str] = None, parameters_normalized: bool = False) -> dict:
+    def invoke_workflow(
+        self,
+        workflow_id: str,
+        inputs: Optional[dict] = None,
+        params: Optional[dict] = None,
+        history_id: Optional[str] = None,
+        history_name: Optional[str] = None,
+        import_inputs_to_history: bool = False,
+        replacement_params: Optional[dict] = None,
+        allow_tool_state_corrections: bool = False,
+        inputs_by: Optional[InputsBy] = None,
+        parameters_normalized: bool = False,
+        require_exact_tool_versions: bool = True,
+    ) -> Dict[str, Any]:
         """
         Invoke the workflow identified by ``workflow_id``. This will
         cause a workflow to be scheduled and return an object describing
@@ -343,6 +365,11 @@ class WorkflowClient(Client):
           ``False``, but when setting ``params`` for a subworkflow, ``True`` is
           required.
 
+        :type require_exact_tool_versions: bool
+        :param require_exact_tool_versions: Whether invocation should fail if
+          Galaxy does not have the exact tool versions. Default is ``True``.
+          Parameter does not any effect for Galaxy versions < 22.05.
+
         :rtype: dict
         :return: A dict containing the workflow invocation describing the
           scheduling of the workflow. For example::
@@ -453,30 +480,31 @@ class WorkflowClient(Client):
         """
         payload: Dict[str, Any] = {}
         if inputs:
-            payload['inputs'] = inputs
+            payload["inputs"] = inputs
 
         if params:
-            payload['parameters'] = params
+            payload["parameters"] = params
 
         if replacement_params:
-            payload['replacement_params'] = replacement_params
+            payload["replacement_params"] = replacement_params
 
         if history_id:
-            payload['history'] = f'hist_id={history_id}'
+            payload["history"] = f"hist_id={history_id}"
         elif history_name:
-            payload['history'] = history_name
+            payload["history"] = history_name
         if not import_inputs_to_history:
-            payload['no_add_to_history'] = True
+            payload["no_add_to_history"] = True
         if allow_tool_state_corrections:
-            payload['allow_tool_state_corrections'] = allow_tool_state_corrections
+            payload["allow_tool_state_corrections"] = allow_tool_state_corrections
         if inputs_by is not None:
-            payload['inputs_by'] = inputs_by
+            payload["inputs_by"] = inputs_by
+        payload["require_exact_tool_versions"] = require_exact_tool_versions
         if parameters_normalized:
-            payload['parameters_normalized'] = parameters_normalized
+            payload["parameters_normalized"] = parameters_normalized
         url = self._invocations_url(workflow_id)
         return self._post(payload, url=url)
 
-    def show_invocation(self, workflow_id, invocation_id):
+    def show_invocation(self, workflow_id: str, invocation_id: str) -> Dict[str, Any]:
         """
         Get a workflow invocation object representing the scheduling of a
         workflow. This object may be sparse at first (missing inputs and
@@ -527,7 +555,7 @@ class WorkflowClient(Client):
         url = self._invocation_url(workflow_id, invocation_id)
         return self._get(url=url)
 
-    def get_invocations(self, workflow_id):
+    def get_invocations(self, workflow_id: str) -> List[Dict[str, Any]]:
         """
         Get a list containing all the workflow invocations corresponding to the
         specified workflow.
@@ -552,7 +580,7 @@ class WorkflowClient(Client):
         url = self._invocations_url(workflow_id)
         return self._get(url=url)
 
-    def cancel_invocation(self, workflow_id, invocation_id):
+    def cancel_invocation(self, workflow_id: str, invocation_id: str) -> Dict[str, Any]:
         """
         Cancel the scheduling of a workflow.
 
@@ -568,7 +596,7 @@ class WorkflowClient(Client):
         url = self._invocation_url(workflow_id, invocation_id)
         return self._delete(url=url)
 
-    def show_invocation_step(self, workflow_id, invocation_id, step_id):
+    def show_invocation_step(self, workflow_id: str, invocation_id: str, step_id: str) -> Dict[str, Any]:
         """
         See the details of a particular workflow invocation step.
 
@@ -599,8 +627,10 @@ class WorkflowClient(Client):
         url = self._invocation_step_url(workflow_id, invocation_id, step_id)
         return self._get(url=url)
 
-    def run_invocation_step_action(self, workflow_id, invocation_id, step_id, action):
-        """ Execute an action for an active workflow invocation step. The
+    def run_invocation_step_action(
+        self, workflow_id: str, invocation_id: str, step_id: str, action: Any
+    ) -> Dict[str, Any]:
+        """Execute an action for an active workflow invocation step. The
         nature of this action and what is expected will vary based on the
         the type of workflow step (the only currently valid action is True/False
         for pause steps).
@@ -625,7 +655,7 @@ class WorkflowClient(Client):
         payload = {"action": action}
         return self._put(payload=payload, url=url)
 
-    def delete_workflow(self, workflow_id):
+    def delete_workflow(self, workflow_id: str) -> str:
         """
         Delete a workflow identified by `workflow_id`.
 
@@ -641,7 +671,9 @@ class WorkflowClient(Client):
         """
         return self._delete(id=workflow_id)
 
-    def refactor_workflow(self, workflow_id, actions, dry_run=False):
+    def refactor_workflow(
+        self, workflow_id: str, actions: List[Dict[str, Any]], dry_run: bool = False
+    ) -> Dict[str, Any]:
         """
         Refactor workflow with given actions.
 
@@ -676,14 +708,20 @@ class WorkflowClient(Client):
                  and the refactored workflow.
         """
         payload = {
-            'actions': actions,
-            'dry_run': dry_run,
+            "actions": actions,
+            "dry_run": dry_run,
         }
-        url = '/'.join((self._make_url(workflow_id), 'refactor'))
+        url = "/".join((self._make_url(workflow_id), "refactor"))
         return self._put(payload=payload, url=url)
 
-    def extract_workflow_from_history(self, history_id, workflow_name,
-                                      job_ids=None, dataset_hids=None, dataset_collection_hids=None):
+    def extract_workflow_from_history(
+        self,
+        history_id: str,
+        workflow_name: str,
+        job_ids: Optional[List[str]] = None,
+        dataset_hids: Optional[List[str]] = None,
+        dataset_collection_hids: Optional[List[str]] = None,
+    ) -> Dict[str, Any]:
         """
         Extract a workflow from a history.
 
@@ -712,11 +750,11 @@ class WorkflowClient(Client):
             "job_ids": job_ids if job_ids else [],
             "dataset_ids": dataset_hids if dataset_hids else [],
             "dataset_collection_ids": dataset_collection_hids if dataset_collection_hids else [],
-            "workflow_name": workflow_name
+            "workflow_name": workflow_name,
         }
         return self._post(payload=payload)
 
-    def show_versions(self, workflow_id):
+    def show_versions(self, workflow_id: str) -> List[Dict[str, Any]]:
         """
         Get versions for a workflow.
 
@@ -726,17 +764,17 @@ class WorkflowClient(Client):
         :rtype: list of dicts
         :return: Ordered list of version descriptions for this workflow
         """
-        url = self._make_url(workflow_id) + '/versions'
+        url = self._make_url(workflow_id) + "/versions"
         return self._get(url=url)
 
-    def _invocation_step_url(self, workflow_id, invocation_id, step_id):
-        return '/'.join((self._invocation_url(workflow_id, invocation_id), "steps", step_id))
+    def _invocation_step_url(self, workflow_id: str, invocation_id: str, step_id: str) -> str:
+        return "/".join((self._invocation_url(workflow_id, invocation_id), "steps", step_id))
 
-    def _invocation_url(self, workflow_id, invocation_id):
-        return '/'.join((self._invocations_url(workflow_id), invocation_id))
+    def _invocation_url(self, workflow_id: str, invocation_id: str) -> str:
+        return "/".join((self._invocations_url(workflow_id), invocation_id))
 
-    def _invocations_url(self, workflow_id):
-        return '/'.join((self._make_url(workflow_id), 'invocations'))
+    def _invocations_url(self, workflow_id: str) -> str:
+        return "/".join((self._make_url(workflow_id), "invocations"))
 
 
-__all__ = ('WorkflowClient',)
+__all__ = ("WorkflowClient",)
