@@ -1,15 +1,15 @@
 """
 This example retrieves details of all the Data Libraries available to us and lists information on them.
 
-Usage: python list_data_libraries.py <galaxy-url> <galaxy-API-key>
+Usage: python3 list_data_libraries.py <galaxy-url> <galaxy-API-key>
 """
-from __future__ import print_function
+
 import sys
 
 from bioblend.galaxy import GalaxyInstance
 
 if len(sys.argv) != 3:
-    print("Usage: python list_data_libraries.py <galaxy-url> <galaxy-API-key>")
+    print("Usage: python3 list_data_libraries.py <galaxy-url> <galaxy-API-key>")
     sys.exit(1)
 galaxy_url = sys.argv[1]
 galaxy_key = sys.argv[2]
@@ -27,4 +27,4 @@ if len(libraries) == 0:
 else:
     print("\nData Libraries:")
     for lib_dict in libraries:
-        print("{0} : {1}".format(lib_dict['name'], lib_dict['id']))
+        print(f"{lib_dict['name']} : {lib_dict['id']}")
