@@ -3,7 +3,7 @@ This example retrieves details of all the Histories in our Galaxy account and li
 
 Usage: python list_histories.py <Galaxy_URL> <Galaxy_API_key>
 """
-from __future__ import print_function
+
 import sys
 
 from bioblend.galaxy.objects import GalaxyInstance
@@ -30,4 +30,4 @@ if len(histories) == 0:
 else:
     print("\nHistories:")
     for hist in histories:
-        print("{0} ({1}) : {2}".format(hist.name, hist.wrapped['nice_size'], hist.id))
+        print(f"{hist.name} ({hist.wrapped['nice_size']}) : {hist.id}")
